def alphad(self, value="", **kwargs):
    """Defines the mass matrix multiplier for damping.

    APDL Command: ALPHAD

    Parameters
    ----------
    value
        Mass matrix multiplier for damping.

    Notes
    -----
    This command defines the mass matrix multiplier α used to form the
    viscous damping matrix [C] = α[M] where [M] is the mass matrix.

    Values of  α may also be input as a material property (use the ALPD
    label on the MP command). If ALPD is included, the ALPD value is added
    to the ALPHAD value as appropriate (see Damping Matrices in the
    Mechanical APDL Theory Reference). Damping is not used in the static
    (ANTYPE,STATIC) or buckling (ANTYPE,BUCKLE) analyses.

    This command is also valid in PREP7.
    """
    command = f"ALPHAD,{value}"
    return self.run(command, **kwargs)


def betad(self, value="", **kwargs):
    """Defines the stiffness matrix multiplier for damping.

    APDL Command: BETAD

    Parameters
    ----------
    value
        Stiffness matrix multiplier for damping.

    Notes
    -----
    This command defines the stiffness matrix multiplier β used to form the
    viscous damping matrix [C] = β [K] where [K] is the stiffness matrix.

    Values of : β may also be input as a material property (use the BETD
    label on the MP command).  If BETD is included, the BETD value is added
    to the BETAD value as appropriate (see Damping Matrices in the
    Mechanical APDL Theory Reference).  Damping is not used in the static
    (ANTYPE,STATIC) or buckling (ANTYPE,BUCKLE) analyses.

    This command is also valid in PREP7.
    """
    command = f"BETAD,{value}"
    return self.run(command, **kwargs)


def dmprat(self, ratio="", **kwargs):
    """Sets a constant modal damping ratio.

    APDL Command: DMPRAT

    Parameters
    ----------
    ratio
        Modal damping ratio (for example, 2% is input as 0.02).

    Notes
    -----
    Sets a constant damping ratio for use in the mode-superposition
    transient (ANTYPE,TRANS) or harmonic (ANTYPE,HARMIC) analysis and the
    spectrum (ANTYPE,SPECTR) analysis.

    This command is also valid in PREP7.
    """
    command = f"DMPRAT,{ratio}"
    return self.run(command, **kwargs)


def dmpstr(self, coeff="", **kwargs):
    """Sets a constant structural damping coefficient.

    APDL Command: DMPSTR

    Parameters
    ----------
    coeff
        Structural damping coefficient.

    Notes
    -----
    Sets a constant structural (or hysteretic) damping coefficient for use
    in harmonic (ANTYPE,HARMIC) analyses (FULL, MSUP, and VT) and modal
    analyses (ANTYPE,MODAL with MODOPT,UNSYM, DAMP or QRDAMP).

    Note that for structures with multiple materials, MP,DMPR can also be
    used to specify constant structural material damping on a per material
    basis. Note that if both DMPSTR and MP,DMPR are specified, the damping
    effects are additive.

    Caution:: : DMPSTR adds the damping contribution as gK, whereas MP,DMPR
    adds the contribution on a per-material basis as 2gK. For more
    information, see Damping Matrices in the Mechanical APDL Theory
    Reference.

    This command is also valid in PREP7.
    """
    command = f"DMPSTR,{coeff}"
    return self.run(command, **kwargs)


def frqscl(self, scaling="", **kwargs):
    """Turns on automatic scaling of the entire mass matrix and frequency

    APDL Command: FRQSCL
    range for modal analyses using the Block Lanczos, PCG Lanczos, or
    Supernode mode extraction method.

    Parameters
    ----------
    scaling


        Off  - Do not use automatic scaling of the mass matrix and frequency range.

        On  - Use automatic scaling of the mass matrix and frequency range.

    Notes
    -----
    Use this command to deactivate or force activation of automatic scaling
    of the entire mass matrix and frequency range for modal analyses where
    the entire mass matrix is significantly different (i.e., orders of
    magnitude difference) than the entire stiffness matrix (for example,
    due to the particular unit system being used).  Where the mass matrix
    is significantly smaller compared to the stiffness matrix, the
    eigenvalues will tend to approach very large numbers (>10e12), making
    the Block Lanczos, PCG Lanczos, or Supernode mode extraction method
    less efficient and more likely to miss modes.

    ANSYS uses scaling (if appropriate) by default. However, you can issue
    FRQSCL,ON to force the entire mass matrix and frequency range to be
    scaled to bring the stiffness and mass matrices closer together in
    terms of orders of magnitude, improving efficiency and reducing the
    likelihood of missed modes.  The resulting eigenvalues are then
    automatically scaled back to the original system. If you are using
    micro MKS units, where the density is typically very small compared to
    the stiffness, you may want to issue FRQSCL,ON to force scaling on.

    If the stiffness and mass are on the same scale, FRQSCL,ON has no
    effect.

    This command is available only for modal analyses using the Block
    Lanczos, PCG Lanczos, or Supernode mode extraction method (MODOPT,LANB,
    LANPCG, or SNODE).

    This command is not valid and has no effect when used in conjunction
    with the MSAVE,ON command in a modal analysis with the PCG Lanczos mode
    extraction method.
    """
    command = f"FRQSCL,{scaling}"
    return self.run(command, **kwargs)


def harfrq(self, freqb="", freqe="", logopt="", freqarr="", toler="",
           **kwargs):
    """Defines the frequency range in a harmonic analysis.

    APDL Command: HARFRQ

    Parameters
    ----------
    freqb
        Frequency (Hz) at the beginning of the FREQB to FREQE range (if
        FREQE > FREQB).  If FREQE is blank, the solution is done only at
        frequency FREQB (the central frequency of octave bands, when LogOpt
        = OB1, OB2, OB3, OB6, OB12 or OB24).

    freqe
        Frequency at end of this range.  Solutions are done at an interval
        of (FREQE-FREQB) / NSBSTP, ending at FREQE.  No solution is done at
        the beginning of the frequency range.  NSBSTP is input via the
        NSUBST command.  See the EXPSOL command documentation for expansion
        pass solutions.

    --
        Reserved.

    logopt
        Logarithm frequency span. Solutions are done at an interval of
        (log(FREQE) - log(FREQB)) / (NSBSTP-1), (NSBSTP>1). The central
        frequency or beginning frequency is used for NSBSTP = 1. Valid
        values are:

        OB1 - Octave band.

        OB2 - 1/2 octave band.

        OB3 - 1/3 octave band.

        OB6 - 1/6 octave band.

        OB12 - 1/12 octave band.

        OB24 - 1/24 octave band.

        LOG - General logarithm frequency span.

    freqarr
        An array containing frequency values (Hz). Combined with the
        tolerance argument, Toler, these values are merged with values
        calculated based on the specifications from FREQB, FREQE, and
        LogOpt, as well NSBSTP on the NSUBST command and Clust on the HROUT
        command. Enclose the array name in percent (%) signs (for example,
        HARFRQ,,,,,%arrname%). Use *DIM to define the array.

    toler
        Tolerance to determine if a user input frequency value in FREQARR
        is a duplicate and can be ignored. Two frequency values are
        considered duplicates if their difference is less than the
        frequency range multiplied by the tolerance. The default value is 1
        x 10-5.

    Notes
    -----
    Defines the frequency range for loads in the harmonic analysis
    (ANTYPE,HARMIC).

    Do not use this command for a harmonic ocean wave analysis (HROCEAN).

    When frequencies are user-defined, the array FREQARR must be one-
    dimensional and contain positive values. User-defined frequency input
    is not supported in the following cases:

    in a cyclic symmetry harmonic analysis

    when the Variational Technology method is used (Method = VT on the
    HROPT command)

    This command is also valid in PREP7.
    """
    command = f"HARFRQ,{freqb},{freqe},{logopt},{freqarr},{toler}"
    return self.run(command, **kwargs)


def hrexp(self, angle="", **kwargs):
    """Specifies the phase angle for the harmonic analysis expansion pass.

    APDL Command: HREXP

    Parameters
    ----------
    angle
        Phase angle (degrees) for expansion pass.  If ALL (default), use
        both 0.0° (real) and 90.0° (imaginary) phase angles.

    Notes
    -----
    Specifies the phase angle where the expansion pass will be done for a
    harmonic mode-superposition expansion pass.

    For a specific angle, the following real solution is stored in the
    results (*.rst) file:

    Where:

    i is the degree of freedom number.

     is the amplitude of the i th degree of freedom solution

    Φi is the phase shift angle of the i th degree of freedom solution

    ϕ is the supplied phase shift angle (ANGLE)

    If ANGLE is ALL, both the real and imaginary parts of the solution are
    stored in the results file.

    For more details about the solution equations, see Harmonic Analyses in
    the Mechanical APDL Theory Reference.

    This command is ignored if the HROPT command has been issued with
    Method = VT or Method = VTRU.

    This command is also valid in PREP7.
    """
    command = f"HREXP,{angle}"
    return self.run(command, **kwargs)


def hrocean(self, type_="", nphase="", **kwargs):
    """Perform the harmonic ocean wave procedure (HOWP).

    APDL Command: HROCEAN

    Parameters
    ----------
    type\_
        Specifies how to include ocean wave information in a harmonic
        analysis:

        HARMONIC - Performs a harmonic analysis using both real and imaginary load vectors
                   calculated via the harmonic ocean wave procedure (HOWP).
                   This behavior is the default. This option performs a
                   harmonic analysis running at a frequency determined by
                   the wave period (specified via OCTABLE command input).

        STATIC - Performs a static analysis using both real and imaginary load vectors
                 (calculated via HOWP). This option works by performing a
                 harmonic analysis running at a frequency of 0.0.

        OFF - Deactivates a previously activated HOWP and performs a standard harmonic
              analysis.

    nphase
        Positive number specifying the number of phases to calculate
        forces. This value must be at least 8. The default value is 20.

    Notes
    -----
    The HROCEAN command applies ocean wave information (obtained via the
    OCDATA and OCTABLE commands) in a harmonic analysis (ANTYPE,HARMIC) as
    real and imaginary forces.

    You can apply only one ocean load at a time.

    The applied frequency in the harmonic (Type = HARMONIC) analysis is
    based on the wave period input on the OCTABLE command (and not on
    HARFRQ command input, which cannot be used). Phase-shift input on the
    OCTABLE command is ignored.

    HOWP does not generate a damping matrix. If you require a damping
    matrix, you must add it separately.

    The command applies to regular wave types only (Airy with one wave
    component, Wheeler with one wave component, Stokes, and stream
    function). Irregular wave types are not supported. For information
    about wave types, see Hydrodynamic Loads in the Mechanical APDL Theory
    Reference.

    The program calculates the forces on each load component of each
    element at NPHASE solutions, spread evenly over one wave cycle. Then,
    the minimum and maximum, and the phase between them, are calculated.
    The command uses the resulting information to generate the real and
    imaginary loads.

    HOWP cannot be used with stress stiffening.

    HOWP works with the full harmonic analysis method (HROPT,FULL) only.

    For more information, see Harmonic Ocean Wave Procedure (HOWP) in the
    Mechanical APDL Theory Reference.

    This command is also valid in PREP7.
    """
    command = f"HROCEAN,{type_},{nphase}"
    return self.run(command, **kwargs)


def hropt(self, method="", maxmode="", minmode="", mcout="", damp="",
          **kwargs):
    """Specifies harmonic analysis options.

    APDL Command: HROPT

    Parameters
    ----------
    method
        Solution method for the harmonic analysis:

        AUTO - Automatically select the most efficient method. Either the FULL method or the
               Variational Technology method is selected depending on the
               model. (default method).

        FULL - Full method.

        MSUP - Mode-superposition method.

        VT - Variational Technology method (based on FULL harmonic algorithm).

        VTPA - Variational Technology perfect absorber method (based on FULL harmonic
               algorithm).

        VTRU - Variational Technology reuse method (based on FULL harmonic algorithm).

    maxmode
        Largest mode number to be used to calculate the response (for
        Method = MSUP only).  Defaults to the highest mode calculated in
        the preceding modal analysis.

    minmode
        Smallest mode number to be used (for Method = MSUP only).  Defaults
        to 1.

    mcout
        Modal coordinates output key (valid only for the mode superposition
        method MSUP):

        NO - No output of modal coordinates (default).

        YES - Output modal coordinates to the text file jobname.MCF.

    damp
        Damping mode for frequency-dependent material properties (valid
        only for the Variational Technology Method VT).

        Hysteretic - Not proportional to the frequency.

        Viscous - Proportional to the frequency (default).

    Notes
    -----
    Specifies the method of solution for a harmonic analysis
    (ANTYPE,HARMIC).  If used in SOLUTION, this command is valid only
    within the first load step. See the product restrictions indicated
    below.

    For cyclic symmetry mode-superposition harmonic solutions, MAXMODE and
    MINMODE are ignored.

    To include residual vectors in your mode-superposition harmonic
    analysis, specify RESVEC,ON.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: The VTRU method is not supported.
    """
    command = f"HROPT,{method},{maxmode},{minmode},{mcout},{damp}"
    return self.run(command, **kwargs)


def hrout(self, reimky="", clust="", mcont="", **kwargs):
    """Specifies the harmonic analysis output options.

    APDL Command: HROUT

    Parameters
    ----------
    reimky
        Real/Imaginary print key:

        ON - Print complex displacements as real and imaginary components (default).

        OFF - Print complex displacements as amplitude and phase angle (degrees).

    clust
        Cluster option (for HROPT,MSUP):

        OFF - Uniform spacing of frequency solutions (default).

        ON - Cluster frequency solutions about natural frequencies.

    mcont
        Mode contributions key (for HROPT,MSUP):

        OFF - No print of mode contributions at each frequency (default).

        ON - Print mode contributions at each frequency.

    Notes
    -----
    Specifies the harmonic analysis (ANTYPE,HARMIC) output options.  If
    used in SOLUTION, this command is valid only within the first load
    step. OUTPR,NSOL must be specified to print mode contributions at each
    frequency.

    This command is ignored if the HROPT command has been issued with
    Method = VT, VTPA, or VTRU. Displacements are not available at expanded
    frequencies with these solution methods.

    For cyclic symmetry mode-superposition harmonic solutions, the cluster
    option is not available.

    This command is also valid in PREP7.
    """
    command = f"HROUT,{reimky},{clust},{mcont}"
    return self.run(command, **kwargs)


def lvscale(self, fact="", ldstep="", **kwargs):
    """Scales the load vector for mode-superposition analyses.

    APDL Command: LVSCALE

    Parameters
    ----------
    fact
        Scale factor applied to both the real and imaginary (if they exist)
        components of the load vector. Defaults to 0.0.

    ldstep
        Specifies the load step number from the modal analysis
        (MODCONT,ON). It corresponds to the load vector number. Defaults to
        1. The maximum value is 240.

    Notes
    -----
    Specifies the scale factor for the load vector that was created in a
    modal (ANTYPE,MODAL) analysis.  Applies only to the mode-superposition
    transient analysis (ANTYPE,TRANS), mode-superposition harmonic analysis
    (ANTYPE,HARMIC), random vibration analysis (ANTYPE,SPECTR with
    SPOPT,PSD), and multiple point response spectrum analysis
    (ANTYPE,SPECTR with SPOPT,MPRS).  For PSD and MPRS analyses, LVSCALE is
    only applicable for pressure loading.

    The LVSCALE command supports tabular boundary conditions (%TABNAME_X%)
    for FACT input values only as a function of time in the mode-
    superposition transient (ANTYPE,TRANS) or as a function of frequency in
    mode-superposition harmonic (ANTYPE,HARMIC).

    MPC contact generates constraint equations that can include constant
    terms (included on the right-hand side of the system equation). The
    LVSCALE command scales the constant terms.

    In mode-superposition transient and harmonic analyses, all of the load
    vectors need to be scaled in the first load step. Use a zero scale
    factor if they are not actually used in this first load step. :
    Similarly, in random vibration and multipoint response spectrum
    analyses, all of the load vectors need to be scaled in the first
    participation factor calculation (PFACT). : Use a zero scale factor if
    they are not actually used for the first input table.

    This command is also valid in PREP7.
    """
    command = f"LVSCALE,{fact},{ldstep}"
    return self.run(command, **kwargs)


def mascale(self, massfact="", **kwargs):
    """Activates scaling of the entire system matrix.

    APDL Command: MASCALE

    Parameters
    ----------
    massfact
       Scaling factor (> 0) for the mass matrix. Default = 1.0.

    Notes
    -----
    This command is supported in the first load step of the analysis only.
    The following features are not affected by the scaling:

    * Ocean loading
    * Steady-state rolling SSTATE

    The mass-related information (mass, center of mass, and mass
    moments of inertia) printed in the mass summary is based on
    unscaled mass properties.
    """
    return self.run(f"MASCALE,{massfact}", **kwargs)


def mdamp(self, stloc="", v1="", v2="", v3="", v4="", v5="", v6="",
          **kwargs):
    """Defines the damping ratios as a function of mode.

    APDL Command: MDAMP

    Parameters
    ----------
    stloc
        Starting location in table for entering data.  For example, if
        STLOC = 1, data input in the V1 field applies to the first constant
        in the table.  If STLOC = 7, data input in the V1 field applies to
        the seventh constant in the table, etc.  Defaults to the last
        location filled + 1.

    v1, v2, v3, . . . , v6
        Data assigned to six locations starting with STLOC.  If a value is
        already in this location, it will be redefined.  Blank values for
        V2 to V6 leave the corresponding previous value unchanged.

    Notes
    -----
    Defines the damping ratios as a function of mode.  Table position
    corresponds to mode number.  These ratios are added to the DMPRAT
    value, if defined.  Use STAT command to list current values.  Applies
    to the mode-superposition harmonic (ANTYPE,HARMIC), the mode-
    superposition linear transient dynamic (ANTYPE,TRANS), and the spectrum
    (ANTYPE,SPECTR) analyses.  Repeat MDAMP command for additional
    constants (10000 maximum).

    MDAMP can also be defined in a substructure analysis using component
    mode synthesis with fixed-interface method (ANTYPE,SUBSTR with
    CMSOPT,FIX and SEOPT,,,3). The damping ratios are added to the diagonal
    of the reduced damping matrix as explained in Component Mode Synthesis
    (CMS).

    This command is also valid in PREP7.
    """
    command = f"MDAMP,{stloc},{v1},{v2},{v3},{v4},{v5},{v6}"
    return self.run(command, **kwargs)


def mdplot(self, function="", dmpname="", scale="", **kwargs):
    """Plots frequency-dependent modal damping coefficients calculated by

    APDL Command: MDPLOT
    DMPEXT.

    Parameters
    ----------
    function
        Function to display.

        d_coeff - Damping coefficient

        s_coeff - Squeeze coefficient

        d_ratio - Damping ratio

        s_ratio - Squeeze stiffness ratio

    dmpname
        Array parameter name where damping information is stored. Defaults
        to d_damp.

    scale
        Indicates whether to perform a linear or a double logarithmic plot.

        LIN - Perform a linear plot. Default

        LOG - Perform a double logarithmic plot.

    Notes
    -----
    See Thin Film Analysis for more information on thin film analyses.
    """
    command = f"MDPLOT,{function},{dmpname},{scale}"
    return self.run(command, **kwargs)


def midtol(self, key="", tolerb="", resfq="", **kwargs):
    """Sets midstep residual criterion values for structural transient

    APDL Command: MIDTOL
    analyses.

    Parameters
    ----------
    key
        Midstep residual criterion activation key.

        ON or 1 - Activate midstep residual criterion in a structural transient analysis
                  (default).

        OFF or 0 - Deactivate midstep residual criterion in a structural transient analysis.

        STAT  - List the current midstep residual criterion setting.

    tolerb
        Midstep residual tolerance or reference value for bisection.
        Defaults to 100 times the TOLER setting of the CNVTOL command.

    resfq
        Key to use response frequency computation along with midstep
        residual criterion for automatic time stepping (AUTOTS,ON).

        OFF or 0 - Do not calculate response frequency and do not consider it in the automatic
                   time stepping (default).

        ON or 1 - Calculate response frequency and consider it in the automatic time stepping.

    Notes
    -----
    When TOLERB is input as a tolerance value (TOLERB > 0), the typical
    force and/or moment from the regular time step is used in the midstep
    residual force and/or moment comparison.

    In a structural transient analysis, the suggested tolerance range of
    TOLERB (TOLERB > 0) is as follows:

    If the structural transient analysis is elastic and linear, and the
    load is constant or changes slowly, use a smaller value of TOLERB to
    achieve an accurate solution. If the analysis involves large amounts of
    energy dissipation, such as elastic-plastic material, TOLERB can be
    larger. If the analysis includes contact or rapidly varying loads, a
    smaller value of TOLERB should be used if high frequency response is
    important; otherwise, a larger value of TOLERB may be used to enable
    faster convergence with larger time step sizes.

    For more information on how the midstep criterion is used by the
    program, see Midstep Residual for Structural Dynamic Analysis in the
    Mechanical APDL Theory Reference.

    This command is also valid in PREP7.
    """
    command = f"MIDTOL,{key},{tolerb},{resfq}"
    return self.run(command, **kwargs)


def modcont(self, mlskey="", enforcedkey="", **kwargs):
    """Specify additional modal analysis options.

    APDL Command: MODCONT

    Parameters
    ----------
    mlskey
        Multiple load step key:

        OFF - Perform the modal analysis (compute the eigenvalues and the load vector) for
              each load step. (default)

        ON - Perform the modal analysis (compute the eigenvalues and the load vector) only
             for the first load step; form the load vector for each
             subsequent load step (without repeating the eigenvalue
             calculations) and write all load vectors to the Jobname.MODE
             file for downstream mode-superposition analyses.

    enforcedkey
        Enforced motion key:

        OFF - Do not calculate enforced static modes. (default)

        ON - Calculate enforced static modes and write them to the Jobname.MODE file.

    Notes
    -----
    Specifies additional modal analysis (ANTYPE,MODAL) options.

    Use the LVSCALE command to apply the desired load in a mode-
    superposition transient or harmonic analysis.

    The maximum number of load vectors that can be used in the downstream
    mode-superposition transient or harmonic analysis is: 240.

    Generation of multiple loads (MLSkey = ON) is supported by the Block
    Lanczos, PCG Lanczos, Supernode, Subspace, Unsymmetric, and QR damped
    modal methods.

    The enforced motion calculation (EnforcedKey = ON) is supported by the
    Block Lanczos and Supernode mode extraction methods.
    """
    command = f"MODCONT,{mlskey},{enforcedkey}"
    return self.run(command, **kwargs)


def modseloption(self, dir1="", dir2="", dir3="", dir4="", dir5="",
                 dir6="", **kwargs):
    """APDL Command: MODSELOPTION

    Parameters
    ----------
    dir1, dir2, dir3, dir4, dir5, dir6
        Selection of the direction to be expanded.

        For ``modeselmethod=effm`` on the MXPAND command, the
        directions correspond to the global Cartesian directions,
        i.e. 1=X, 2=Y, 3=Z, 4=ROTX, 5=ROTY, and 6=ROTZ. If dir1 = YES,
        then any mode in this direction is expanded if its modal
        effective mass divided by the total mass (modal effective mass
        ratio) is greater than SIGNIF on the MXPAND command. If
        dir1=NO, then the specified direction is not considered as a
        criterion for expansion. If dir1 is given a numerical decimal
        value, modes in that direction are selected (starting from the
        ones with the largest modal effective mass ratios to the
        smallest) until the sum of their modal effective mass ratio
        equals this requested threshold.

        For ModeSelMethod = MODC on the MXPAND command, dir1
        corresponds to the first input spectrum, dir2 to the second,
        etc. (i.e. for multiple spectrum inputs; the actual directions
        correspond to their respective SED directions). If dir1=YES,
        then any mode in this spectrum is ex- panded if its mode
        coefficient divided by the largest mode coefficient is greater
        than SIGNIF on the MXPAND command. If dir1=NO, then the
        specified direction is not considered as a criterion for
        expansion.

    Notes
    -----
    This command is only applicable when a mode selection method is defined
    (ModeSelMethod on the MXPAND command). See Using Mode Selection in the
    Mechanical APDL Structural Analysis Guide for more details.

    If a numerical value is specified for a direction, the significance
    threshold (SIGNIF on the MXPAND command) is ignored for the selection
    of the modes in this direction.

    If a mode is determined to be expanded in any of the 6 directions, it
    will be expanded in the .MODE file. : Otherwise, the mode will not be
    expanded.

    The default behavior is to consider all directions for expansion.
    """
    command = f"MODSELOPTION,{dir1},{dir2},{dir3},{dir4},{dir5},{dir6}"
    return self.run(command, **kwargs)


def modopt(self, method="", nmode="", freqb="", freqe="", cpxmod="",
           nrmkey="", modtype="", blocksize="", freqmod="", **kwargs):
    """Specifies modal analysis options.

    APDL Command: MODOPT

    Parameters
    ----------
    method
        Mode-extraction method to be used for the modal analysis.

        LANB - Block Lanczos

        LANPCG - PCG Lanczos

        SNODE - Supernode modal solver

        SUBSP - Subspace algorithm

        UNSYM - Unsymmetric matrix

        DAMP - Damped system

        QRDAMP - Damped system using QR algorithm

        VT - Variational Technology

    nmode
        The number of modes to extract. The value can depend on the value
        supplied for Method. NMODE has no default and must be specified. If
        Method = LANB, LANPCG, or SNODE, the number of modes that can be
        extracted can equal the DOFs in the model after the application of
        all boundary conditions.

    freqb
        The beginning, or lower end, of the frequency range of interest.

    freqe
        The ending, or upper end, of the frequency range of interest (in
        Hz). The default for Method = SNODE is described below. The default
        for all other methods is to calculate all modes, regardless of
        their maximum frequency.

    cpxmod
        Complex eigenmode key. (Valid only when Method = QRDAMP or Method =
        UNSYM).

        AUTO - Determine automatically if the eigensolutions are real or complex and output
               them accordingly. This is the default for Method = UNSYM.
               Not supported for Method = QRDAMP.

        ON or CPLX - Calculate and output complex eigenmode shapes.

        OFF or REAL - Do not calculate complex eigenmode shapes. This is required if a mode-
                      superposition analysis is intended after the modal
                      analysis for Method = QRDAMP. This is the default for
                      this method.

    nrmkey
        Mode shape normalization key:

        OFF - Normalize the mode shapes to the mass matrix (default).

        ON - Normalize the mode shapes to unity instead of to the mass matrix.  If a
             subsequent spectrum or mode-superposition analysis is planned,
             the mode shapes should be normalized to the mass matrix
             (Nrmkey = OFF).

    modtype
        Type of modes calculated by the eigensolver. Only applicable to the
        unsymmetric eigensolver.

        Blank - Right eigenmodes. This value is the default.

        BOTH - Right and left eigenmodes. The left eigenmodes are written to Jobname.LMODE.
               This option must be activated if a mode-superposition
               analysis is intended.

    blocksize
        The block vector size to be used with the Block Lanczos or
        Subspace eigensolver (used only when Method = LANB or
        SUBSP). BlockSize must be an integer value between 0 and
        16. When BlockSize = zero or blank, the code decides the
        block size internally (normally, a value of 8 is used for
        LANB and a value of 6 is used for SUBSP).  Typically,
        higher BlockSize values are more efficient under each of
        the following conditions:

        - When running in out-of-core mode and there is not enough
          physical memory to buffer all of the files written by
          the Block Lanczos or Subspace eigensolver (and thus, the
          time spent doing I/O is considerable).
        - Many modes are requested (>100).
        - Higher-order solid elements dominate the model.

        The memory usage only slightly increases as BlockSize is
        increased. It is recommended that you use a value
        divisible by 4 (4, 8, 12, or 16).

    freqmod
        The specified frequency when the solved eigenvalues are no
        longer frequencies (for example, the model has the Floquet
        periodic boundary condition). In a modal analysis, the
        Floquet periodic boundary condition (body load FPBC) is
        only valid for the acoustic elements FLUID30, FLUID220,
        and FLUID221.

    Notes
    -----
    Specifies modal analysis (ANTYPE,MODAL) options. Additional options
    used only for the Supernode (SNODE) eigensolver are specified by the
    SNOPTION command. Additional options used only for the Subspace (SUBSP)
    eigensolver are specified by the SUBOPT command. If Method = LANPCG,
    ANSYS automatically switches to the PCG solver internally for this
    modal analysis. You can further control the efficiency of the PCG
    solver with the PCGOPT and EQSLV commands.

    For models that involve a non-symmetric element stiffness matrix, as in
    the case of a contact element with frictional contact, the QRDAMP
    eigensolver (MODOPT, QRDAMP) extracts modes in the modal subspace
    formed by the eigenmodes from the symmetrized eigenproblem. The QRDAMP
    eigensolver symmetrizes the element stiffness matrix on the first pass
    of the eigensolution, and in the second pass, eigenmodes are extracted
    in the modal subspace of the first eigensolution pass. For such non-
    symmetric eigenproblems, you should verify the eigenvalue and eigenmode
    results using the non-symmetric matrix eigensolver  (MODOPT,UNSYM).

    The DAMP and QRDAMP options cannot be followed by a subsequent spectrum
    analysis. The UNSYM method supports spectrum analysis when
    eigensolutions are real.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: The VT extraction method is not
    supported in Distributed ANSYS. All other extraction methods are
    supported. However, PCG Lanczos, SUBSP, UNSYM, DAMP, and QRDAMP are the
    only distributed eigensolvers that will run a fully distributed
    solution. The Block Lanczos and Supernode eigensolvers are not
    distributed eigensolvers; therefore, you will not see the full
    performance improvements with these methods that you would with a fully
    distributed solution.
    """
    command = f"MODOPT,{method},{nmode},{freqb},{freqe},{cpxmod},{nrmkey},{modtype},{blocksize},,,,{freqmod}"
    return self.run(command, **kwargs)


def mxpand(self, nmode="", freqb="", freqe="", elcalc="", signif="",
           msupkey="", modeselmethod="", **kwargs):
    """Specifies the number of modes to expand and write for a modal or

    APDL Command: MXPAND
    buckling analysis.

    Parameters
    ----------
    nmode
        Number of modes or array name (enclosed in percent signs) to expand
        and write. If blank or ALL, expand and write all modes within the
        frequency range specified. If -1, do not expand and do not write
        modes to the results file during the analysis. If an array name is
        input, the array must contain 1 for the expanded modes and zero
        otherwise, where the array index corresponds to the mode number. To
        specify an array containing the individual modes to expand, enclose
        the array name in percent (%) signs (for example,
        MXPAND,%arrname%). Use the *DIM command to define the array.

    freqb
        Beginning, or lower end, of frequency range of interest. If FREQB
        and FREQE are both blank, expand and write the number of modes
        specified without regard to the frequency range. Defaults to the
        entire range.

    freqe
        Ending, or upper end, of frequency range of interest.

    elcalc
        Element calculation key:

        NO - Do not calculate element results, reaction forces, and energies (default).

        YES - Calculate element results, reaction forces, energies, and the nodal degree of
              freedom solution.

    signif
        Expand only those modes whose significance level exceeds the SIGNIF
        threshold (only applicable when ModeSelMethod is defined).

    msupkey
        Element result superposition key:

        NO - Do not write element results to the mode file Jobname.MODE.

        YES - Write element result to the mode file for use in the expansion pass of a
              subsequent mode-superposition PSD, transient, or harmonic
              analysis (default if Elcalc = YES and the mode shapes are
              normalized to the mass matrix).

    modeselmethod
        Methods for mode selection (not supported for complex
        eigensolvers):

        blank - No mode selection is performed (default).

        MODM - The mode selection is based on the modal effective masses.

        MODC - The mode selection is based on the mode coefficients.

        DDAM - The mode selection is based on DDAM procedure (see Mode Selection Based on DDAM
               Procedure in the Mechanical APDL Structural Analysis Guide
               for more information). This option is applicable only to
               DDAM spectrum analysis.

    Notes
    -----
    Specifies the number of modes to expand and write over a frequency
    range for a modal (ANTYPE,MODAL) or buckling (ANTYPE,BUCKLE) analysis.
    If used in SOLUTION, this command is valid only within the first load
    step.

    There is no limit on the number of expanded modes (NMODE). However,
    there is a limit on the maximum number of modes used via the *GET,,MODE
    command, mode combinations, and the MDAMP command.

    With MSUPkey = YES, the computed element results (Elcalc = YES) are
    written to Jobname.MODE for use in subsequent downstream mode-
    superposition analyses, including harmonic, transient, and PSD
    analyses. This significantly reduces computation time for the
    combination or expansion passes. For limitations, see Option: Number of
    Modes to Expand (MXPAND) in the Mechanical APDL Structural Analysis
    Guide.

    If a mode selection method (ModeSelMethod) is defined, only the
    selected modes will be expanded. See Using Mode Selection in the
    Mechanical APDL Structural Analysis Guide for more details about the
    procedure.

    For array input (NMODE), the array must be dimensioned to be the size
    of the number of modes extracted (NMODE on the MODOPT command). A value
    of 1 in the array indicates the mode is to be expanded, and a value of
    0 indicates not to expand the mode. For the DAMP modal solution, the
    modes are in pairs, so be sure to verify that both modes of a pair have
    the same value. (For example, if modes #3 and #4 are a pair, indices 3
    and 4 in the array should have the same value, 0 or 1.)

    For linear perturbation modal analyses, you must set both Elcalc and
    MSUPkey to YES so that the downstream stress expansion pass can produce
    a solution consistent with the linear or nonlinear base (static or full
    transient) analysis. The prestressed nonlinear element history (saved
    variables) is accessible only in the first and second phases of the
    linear perturbation. The downstream MSUP or PSD analysis can only reuse
    the nonlinear information contained in the Jobname.MODE file that is
    generated in the linear perturbation.

    In a Distributed ANSYS analysis, you must issue MXPAND to specify the
    number of modes to expand when computing the modes and mode shapes. In
    a Distributed ANSYS run, MXPAND cannot be issued in an expansion pass
    (EXPASS).

    This command is also valid in PREP7.
    """
    command = f"MXPAND,{nmode},{freqb},{freqe},{elcalc},{signif},{msupkey},{modeselmethod}"
    return self.run(command, **kwargs)


def qrdopt(self, reusekey="", symmeth="", cmccoutkey="", **kwargs):
    """Specifies additional QRDAMP modal analysis options.

    APDL Command: QRDOPT

    Parameters
    ----------
    reusekey
        Reuse key for method=QRDAMP specified in MODOPT command.

        ON - Reuse the symmetric eigensolution from the previous
             load steps or from the previous solution.

        OFF - Do not reuse (calculates symmetric eigensolution at
              current load step). This is the default.

    symmeth
        Mode-extraction method to be used for the symmetric eigenvalue
        problem.

        LANB - Block Lanczos (default for shared-memory parallel processing).

        SUBSP - Subspace algorithm (default for distributed-memory
                parallel processing).

    cmccoutkey
        Complex Modal Contribution Coefficients (CMCC) output
        key. See Calculate the Complex Mode Contribution
        Coefficients (CMCC) in the Structural Analysis Guide for
        details and usage.

        ON - Output the CMCC to the text file Jobname.CMCC.

        OFF - Do not output the CMCC. This is the default.

    Notes
    -----
    If the filename.modesym file exists in the working directory
    and ReuseKey = ON, filename.modesym will be reused. If
    filename.modesym does not exist in the working directory, the
    symmetric eigensolution will be calculated.

    When ReuseKey=ON, both the new modal analysis
    (filename.modesym usage) and the preceding modal analysis
    (filename.modesym generation) must be performed using the same
    product version number.

    The mode-extraction method changes depending on the type of
    parallelism involved. For performance reasons, the subspace
    method is used with distributed-memory parallel processing
    (Distributed ANSYS) runs, while the Block Lanczos method is
    used with shared-memory parallel processing runs.
    """
    return self.run(f"QRDOPT,{reusekey},,,{symmeth},{cmccoutkey}", **kwargs)


def rigid(self, dof1="", dof2="", dof3="", dof4="", dof5="", dof6="",
          **kwargs):
    """Specifies known rigid body modes (if any) of the model.

    APDL Command: RIGID

    Parameters
    ----------
    dof1, dof2, dof3, . . . , dof6
        Up to six global Cartesian directions of the rigid modes.
        For a completely free 2-D model, use ALL or UX, UY, ROTZ.
        For a completely free 3-D model, use ALL or UX, UY, UZ,
        ROTX, ROTY, ROTZ.  For a constrained model, use UX, UY,
        UZ, ROTX, ROTY, or ROTZ, as appropriate, to specify each
        and every unconstrained direction which exists in the
        model (not specifying every direction may cause
        difficulties in extracting the modes).

    Notes
    -----
    Specifies known rigid body modes (if any) of the model.  This
    command applies only to a component mode synthesis (CMS)
    analysis (see the CMSOPT command).  Any rigid body modes
    specified must be permitted by the applied displacement
    constraints (i.e., do not specify a rigid body mode in a
    constrained direction).  Reissue the command to redefine the
    specification.  If used in SOLUTION, this command is valid
    only within the first load step.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"RIGID,{dof1},{dof2},{dof3},{dof4},{dof5},{dof6}"
    return self.run(command, **kwargs)


def subopt(self, option="", value1="", **kwargs):
    """Specifies Subspace (SUBSP) eigensolver options.

    APDL Command: SUBOPT

    Parameters
    ----------
    option
        One of the following options:

        STRMCK - Controls whether a Sturm sequence check is performed.

        Value1: - OFF

        Do not perform Sturm sequence check (default). - ON

        Perform Sturm sequence check. - MEMORY

        Controls the memory allocation strategy for the Subspace eigensolver. - Value1:

        AUTO - Use the default memory allocation strategy (default).

        INCORE - Force the Subspace eigensolver to allocate in-core memory.

        OUTOFCORE - Force the Subspace eigensolver to use scratch files.

    Notes
    -----
    SUBOPT specifies options to be used with the Subspace eigensolver
    (MODOPT,SUBSP) during a modal analysis.
    """
    command = f"SUBOPT,{option},{value1}"
    return self.run(command, **kwargs)


def timint(self, key="", lab="", **kwargs):
    """Turns on transient effects.

    APDL Command: TIMINT

    Parameters
    ----------
    key
        Transient effects key:

        OFF - No transient effects (static or steady-state).

        ON - Include transient (mass or inertia) effects.

    lab
        Degree of freedom label:

        ALL - Apply this key to all appropriate labels (default).

        STRUC - Apply this key to structural DOFs.

        THERM - Apply this key to thermal DOFs.

        ELECT - Apply this key to electric DOFs.

        MAG - Apply this key to magnetic DOFs.

        FLUID - Apply this key to fluid DOFs.

        DIFFU - Apply this key to concentration of DOFs.

    Notes
    -----
    Indicates whether this load step in a full transient analysis should
    use time integration, that is, whether it includes transient effects
    (e.g. structural inertia, thermal capacitance) or whether it is a
    static (steady-state) load step for the indicated DOFs.  Transient
    initial conditions are introduced at the load step having Key = ON.
    Initial conditions are then determined from the previous two substeps.
    Zero initial velocity and acceleration are assumed if no previous
    substeps exist.  See the Structural Analysis Guide, the Thermal
    Analysis Guide, and the Low-Frequency Electromagnetic Analysis Guide
    for details.

    This command is also valid in PREP7.
    """
    command = f"TIMINT,{key},{lab}"
    return self.run(command, **kwargs)


def tintp(self, gamma="", alpha="", delta="", theta="", oslm="", tol="",
          avsmooth="", alphaf="", alpham="", **kwargs):
    """Defines transient integration parameters.

    APDL Command: TINTP

    Parameters
    ----------
    gamma
        Amplitude decay factor for 2nd order transient integration, e.g.,
        structural dynamics (used only if ALPHA, DELTA, ALPHAF, and ALPHAM
        are blank).  Defaults to 0.005.

    alpha
        2nd order transient integration parameter (used only if GAMMA is
        blank).  Defaults to 0.2525.

    delta
        2nd order transient integration parameter (used only if GAMMA is
        blank).  Defaults to 0.5050.

    theta
        1st order transient (e.g., thermal transient) integration
        parameter.  Defaults to 1.0.

    oslm
        Specifies the oscillation limit criterion for automatic time
        stepping of 1st order transients (e.g., thermal transients).
        Defaults to 0.5 with a tolerance of TOL.

    tol
        Tolerance applied to OSLM.  Defaults to 0.0.

    avsmooth
        Smoothing flag option:

        0 - Include smoothing of the velocity (1st order system)
            or the acceleration (2nd order system) (default).

        1 - Do not include smoothing.

    alphaf
        Interpolation factor in HHT algorithm for force and damping terms
        (used only if GAMMA is blank). Defaults to 0.005.

    alpham
        Interpolation factor in HHT algorithm for inertial term (used only
        if GAMMA is blank). Defaults to 0.0.

    Notes
    -----
    Used to define the transient integration parameters.  For more
    information on transient integration parameters, refer to the
    Mechanical APDL Theory Reference.

    For structural transient analyses, you may choose between the
    Newmark and HHT time integration methods (see the TRNOPT
    command). In this case, if GAMMA is input and the integration
    parameters ALPHA, DELTA, ALPHAF, and ALPHAM are left blank,
    the program will calculate the integration
    parameters. Alternatively, you can input these integration
    parameters directly on this command. However, for the
    unconditional stability and second order accuracy of the time
    integration, these parameters should satisfy a specific
    relationship, as described in Description of Structural and
    Other Second Order Systems of the Mechanical APDL Theory
    Reference.

    In a transient piezoelectric analysis, required input for this
    command is ALPHA = 0.25, DELTA = 0.5, and THETA = 0.5.  For a
    coupled electromagnetic-circuit transient analysis, use THETA
    = 1.0, the default value, to specify the backward Euler
    method.

    This command is also valid in PREP7.
    """
    command = f"TINTP,{gamma},{alpha},{delta},{theta},{oslm},{tol},,,{avsmooth},{alphaf},{alpham}"
    return self.run(command, **kwargs)


def trnopt(self, method="", maxmode="", minmode="", mcfwrite="",
           tintopt="", vaout="", dmpsfreq="", engcalc="", mckey="", **kwargs):
    """Specifies transient analysis options.

    APDL Command: TRNOPT

    Parameters
    ----------
    method
        Solution method for the transient analysis:

        FULL - Full method (default).

        MSUP - Mode-superposition method.

        VT - Variational Technology method.  (Removed by V18.2)

    maxmode
        Largest mode number to be used to calculate the response
        (for Method = MSUP).  Defaults to the highest mode
        calculated in the preceding modal analysis.

    minmode
        Smallest mode number to be used (for Method = MSUP).
        Defaults to 1.

    mcfwrite
        Modal coordinates output key to the .mcf file (valid only
        for the mode-superposition method):

        NO - No output of modal coordinates (default).

        YES - Output modal coordinates to the text file Jobname.MCF.

    tintopt
        Time integration method for the transient analysis:

        NMK or 0 - Newmark algorithm (default).

        HHT or 1 - HHT algorithm (valid only for the full transient method).

    vaout
        Velocities and accelerations output key (valid only for
        mode- superposition transient analysis):

        NO - No output of velocities and accelerations (default).

        YES - Write velocities and accelerations on the reduced displacement file
              Jobname.RDSP.

    dmpsfreq
        Average excitation frequency (Hz) for the calculation of
        equivalent viscous damping from structural damping input
        (DMPSTR and MP,DMPS). See Damping for more
        details. Defaults to zero. If an excitation frequency is
        not specified, structural damping is ignored. If tabular
        excitation frequency data is provided in a full transient
        analysis (DMPSFreqTab on DMPSTR), it supersedes this
        value.

    engcalc
        Additional element energies calculation key:

        NO - Do not calculate additional element energies
        (default).

        YES - Calculate damping energy and work done by external
        loads.

    mckey
        Modal coordinates output key to the .rdsp file (valid only
        for the mode-superposition method):

        AUTO - Writing depends on the modal analysis settings of
        the MXPAND command (default).

        YES - Always write the modal coordinates to the file
        Jobname.rdsp. A subsequent expansion pass (EXPASS) is not
        supported.

    Notes
    -----
    Specifies transient analysis (ANTYPE,TRANS) options. If used
    in SOLUTION, this command is valid only within the first load
    step. Use the TINTP command to set transient integration
    parameters.

    To include residual vectors in your mode-superposition
    transient analysis (Method = MSUP), specify RESVEC,ON.

    Method = MSUP is not available for ocean loading.

    By default in a mode-superposition transient analysis,
    reaction force and other force output contains only static
    contributions. If you want to postprocess the velocities,
    accelerations, and derived results (Lab = TOTAL, DAMP, or
    INERT on the FORCE command), set VAout = YES to activate
    velocity and acceleration output.

    The calculation of additional energies (EngCalc = YES) is
    valid only for the full solution method (Method = FULL). The
    Jobname.ESAV file is always saved in this case. The numerical
    integration for damping energy and work are consistent only if
    solution data are written to the database for every substep
    (OUTRES,ALL,ALL, OUTRES,ESOL,ALL, or OUTRES,VENG, ALL). For
    more information, see Damping Energy and Work Done by External
    Loads in the Mechanical APDL Theory Reference.

    This command is also valid in PREP7.
    """
    command = f"TRNOPT,{method},{maxmode},,{minmode},{mcfwrite},{tintopt},{vaout},{dmpsfreq},{engcalc},{mckey}"
    return self.run(command, **kwargs)



"""These SESSION commands are for file operations, such as deleting,
copying, and listing."""


def anstoasas(self, fname="", key="", **kwargs):
    """Creates an ASAS input file from the current ANSYS model.

    APDL Command: ANSTOASAS

    Parameters
    ----------
    fname
        ASAS file name. Defaults to Jobname.

    key
        Key indicating type of file to produce:

        0 - ASAS file for use by ANSYS Aqwa (no loads written). Creates the file
            Fname.asas.

        1 - ASAS file (all data written, including loads). Creates the file Fname.asas.

        2 - ASAS(NL) file. Creates the file Fname.asnl.

    Notes
    -----
    This command creates an input file for the ANSYS Asas Finite Element
    Analysis System from the model and loads currently in the database,
    based on the currently selected set of elements. Most common structural
    element types are written, as well as sections (or real constants),
    materials, boundary conditions and loads, and solution and load step
    options.

    Data Written

    The following data is written:

    Solution control options

    Nodes

    Elements

    Material data

    Geometry data

    Section data

    ANSYS element components (ASAS sets)

    Boundary conditions

    Loads

    Added mass (via MASS21 element)

    Details are provided in the following sections.

    Not all data is written. You must verify the completeness and accuracy
    of the data. Only loading at the current step is transferred; hence, no
    load step history is captured.

    Solution Control Options

    The ASAS project name is defined as "ANSYS".

    The solution control options are converted as follows:

    JOB: STAT SPIT: KGEOM

    JOB: STAT SPIT: KGEOM

    For conversion to ASAS(NL), the large displacement option is set based
    on NLGEOM, final load solution time is set based on TIME, and sub-step
    times are set based on DELTIM or NSUBST (assuming constant step size).

    Element Data

    If you intend to use the data only with AQWA-WAVE, only the elements
    that form the wetted surface are required.  Selecting these elements
    before invoking the ANSTOASAS command will improve performance.  In
    order for AQWA-WAVE to identify the direction of the wave loading, all
    elements must be defined by nodes in a clockwise direction. For further
    information, refer to the AQWA-WAVE manual.

    The element types are converted as follows:

    SPR1:  SPR2: if: rotational: spring:  FLA2: (ASAS(L): only):  if:
    nodes: are: not: coincident:  and: longitudinal: spring:

    QUM4:  TRM3: -: if: Triangular

    BRK8:  TET4: -: if: Tetrahedral:  BRK6: -: if: Prism

    QUS4:  TBC3: -: if: Triangular

    QUM8:  TRM6: -: if: Triangular

    BR20:  TE10: -: if: Tetrahedral: :  BR15: -: if: Prism

    QUS4:  TBC3: -: if: Triangular

    QUM4:  TRM3: -: if: Triangular

    QUM8:  TRM6: -: if: Triangular

    BRK8:  TET4: -: if: Tetrahedral:  BRK6: -: if: Prism

    BR20:  TE10: -: if: Tetrahedral:  BR15: -: if: Prism

    TCBM: -: if: ASAS(L):  STF4: -: if: ASAS(NL)

    Documentation for this legacy element type appears in the Feature
    Archive.

    Material Data

    Linear isotropic material conversion is supported for ASAS and
    ASAS(NL).

    Geometry Data
    """
    command = "ANSTOASAS,%s,%s" % (str(fname), str(key))
    return self.run(command, **kwargs)


def anstoaqwa(self, fname="", vertaxis="", gc="", rho="", hwl="",
              diffkey="", symxkey="", symykey="", **kwargs):
    """Creates an AQWA-LINE input file from the current ANSYS model.

    APDL Command: ANSTOAQWA

    Parameters
    ----------
    fname
        AQWA file name. Defaults to Jobname.

    vertaxis
        Axis in the vertical direction:

        Y (or 2)  - Global Y axis.

        Z (or 3)  - Global Z axis (default).

    gc
        Gravitational acceleration. Defaults to 9.81.

    rho
        Density of water. Defaults to 1025.0.

    hwl
        Waterline height in model coordinates. Defaults to 0.0.

    diffkey
        Diffracting model key:

        0 - Create a non-diffracting AQWA model.

        1 - Create a diffracting AQWA model (default).

    symxkey
        Key indicating if model is symmetric about the global XZ plane:

        0 - No symmetry about XZ plane (default).

        1 - Use symmetry about XZ plane. Only include (or select) half the model.

    symykey
        Key indicating if model is symmetric about the global YZ plane:

        0 - No symmetry about YZ plane (default).

        1 - Use symmetry about YZ plane. Only include (or select) half the model.

    Notes
    -----
    This command creates the input file Fname.aqwa for the ANSYS Aqwa
    Multi-Body Hydrodynamics System for diffraction analysis in AQWA-LINE
    from the model currently in the database, based on the currently
    selected set of elements. The selected set must only include the hull
    envelope; no internal structure should be selected.

    There should be a line of nodes defined at the waterline. Only those
    elements that are entirely below the waterline will be specified as
    diffracting. If there are no waterline nodes, there will be no
    diffracting elements at the waterline, which will severely reduce the
    accuracy of the diffraction analysis.

    The translator maps PLANE42, SHELL41, SHELL63, and SHELL181 elements to
    PANELs, and maps PIPE16 and PIPE59 elements to TUBEs. It does not
    recognize any other element types. Any material or geometric properties
    can be used for the shell elements, as AQWA does not need any
    properties at all and the command does not use them. All the shell
    elements below the water must have their normals pointing outward.

    TUBE elements in AQWA have material density, outside diameter, wall
    thickness, added mass, and drag coefficients, so appropriate properties
    should be used in the ANSYS model. PIPE59 elements can have added mass
    and damping coefficients; these will be written to the file. The ANSYS
    program uses the inertia coefficient CM, whereas AQWA uses the added
    mass coefficient CA, where CM = (1 + CA). This correction is made
    automatically.

    In AQWA the vertical axis is always the Z-axis. The command can convert
    a model built with either the Y or Z-axis vertical, but the X-axis must
    be horizontal and should preferably be along the fore/aft axis of the
    vessel.  If the structure is symmetric and you wish to use the symmetry
    options, you must only select one half or one quarter of the model, as
    appropriate. If you model a complete vessel and specify X symmetry, the
    AQWA model will contain two sets of coincident elements.

    If you are working from a model created for a structural analysis, it
    will probably be necessary to remesh the model as the structural mesh
    is most likely finer than needed for a diffraction analysis.

    If you enter this command interactively (with the GUI active) and no
    data is provided for the command options, you will be prompted for
    their values.

    You must verify the completeness and accuracy of the data written.
    """
    command = "ANSTOAQWA,%s,%s,%s,%s,%s,%s,%s,%s" % (str(fname), str(vertaxis), str(
        gc), str(rho), str(hwl), str(diffkey), str(symxkey), str(symykey))
    return self.run(command, **kwargs)


def assign(self, ident="", fname="", ext="", lgkey="", **kwargs):
    """Reassigns a file name to an ANSYS file identifier.

    APDL Command: /ASSIGN

    Parameters
    ----------
    ident
        ANSYS file name identifier.  Valid identifiers are: CMS, EMAT,
        EROT,  ESAV, FULL, LN07, LN09, LN11, LN20, LN21, LN22, LN25, LN31,
        LN32, MODE, OSAV, RDSP, RFRQ, RMG, RST, RSTP, RTH, SELD, and SSCR.
        See File Management and Files for file descriptions.  If blank,
        list currently reassigned files.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    lgkey
        Key to specify local or global file name control for the specified
        file identifier in a distributed-memory parallel processing
        (Distributed ANSYS) run. For more information on local and global
        files, see File Handling Conventions in the Parallel Processing
        Guide.

        BOTH - Reassign the file name for both the local and global files (default).

        LOCAL - Reassign the file name for only the local files.

        GLOBAL - Reassign the file name for only the global file.

    Notes
    -----
    The reassignment of file names is valid only if it is done before the
    file is used.  All file reassignments are retained (not cleared) even
    if the database is cleared [/CLEAR] or the Jobname is changed
    [/FILNAME].  Assigned files may be overwritten.  If file name arguments
    (Fname, Ext, --) are blank, the default ANSYS assignment is restored.
    Use SEOPT for SUB files and SEEXP for DSUB files.

    This command is valid only at the Begin Level.

    This command also checks to ensure that the path/file is valid and can
    be written by the user. If it is not valid, an error message will be
    returned. Ensure that the directory exists prior to using /ASSIGN
    command.
    """
    return self.run(f"/ASSIGN,{ident},{fname},{ext},,{lgkey}", **kwargs)


def slashclog(self, fname="", ext="", **kwargs):
    """APDL Command: /CLOG

    Copies the session log file to a named file.

    Parameters
    ----------
    fname
        File name and directory path to which the log file is to be copied
        (248 characters maximum, including directory). If you do not
        specify a directory path, it will default to your working directory
        and you can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    This command is valid in any processor, but only during an interactive
    run.
    """
    command = "/CLOG,%s,%s" % (str(fname), str(ext))
    return self.run(command, **kwargs)


def copy(self, fname1="", ext1="", fname2="", ext2="", distkey="",
         **kwargs):
    """Copies a file.

    APDL Command: /COPY

    Parameters
    ----------
    fname1
        File name to be copied and its directory path (248
        characters maximum for both file name and directory). If
        you do not specify a directory path, it will default to
        your working directory and you can use all 248 characters
        for the file name.

    ext1
        Filename extension (eight-character maximum).

    fname2
        File name to be created and its directory path (248
        characters maximum for both file name and directory). If
        you do not specify a directory path, it will default to
        your working directory and you can use all 248 characters
        for the file name.

    ext2
        Filename extension (eight-character maximum).

    distkey
        Key that specifies whether the copy operation is performed
        on all processes in distributed parallel mode (Distributed
        ANSYS):

        0 (OFF or NO) - The program performs the copy operation
        only on the master process (default).

        1 (ON or YES) - The program performs the copy operation
        locally on each process.

        2 or BOTH - The program performs the copy operation for
        Fname.Ext on the master process and for FnameN.Ext on all
        processes.

    Notes
    -----
    The original file is untouched.  Ex: /COPY,A,,,B copies file A
    to B in the same directory.  /COPY,A,DAT,,,INP copies the file
    A.DAT to A.INP.  See the Operations Guide for details.  ANSYS
    binary and ASCII files can be copied.

    In distributed parallel mode (Distributed ANSYS), only the
    master process will copy Fname1.Ext1 to Fname2.Ext2 by
    default. However, when DistKey is set to 1 (or ON or YES), the
    command is executed by all processes. In this case, Fname1 and
    Fname2 will automatically have the process rank appended to
    them. This means Fname1N.Ext1 will be copied to Fname2N.Ext2
    by all processes, where N is the Distributed ANSYS process
    rank.  For more information see Differences in General
    Behavior in the Parallel Processing Guide.
    """
    command = f"/COPY,{fname1},{ext1},,{fname2},{ext2},,{distkey}"
    return self.run(command, **kwargs)


def fcomp(self, ident="", level="", **kwargs):
    """Specifies file compression level.

    Parameters
    ----------
    ident
        ANSYS file name identifier. Input the label RST to compress
        the following results files: .RST, .RSTP, .RTH, and .RMG. See
        File Management and Files for file descriptions.

    level
        Compression level. Valid input values are 0 (no compression -
        default) to 5 (maximum compression).

    Notes
    -----
    Command Default
    File compression is not performed.

    Specifies file compression for results files (.RST, .RSTP, .RTH,
    and .RMG files). Records are compressed as they are written and
    uncompressed as they are read (for example, by the SET command).

    See File Compression in the Basic Analysis Guide for more details.
    """
    return self.run(f'/FCOMP,{ident},{level}', **kwargs)




def lgwrite(self, fname="", ext="", kedit="", **kwargs):
    """Writes the database command log to a file.

    APDL Command: LGWRITE

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

        The file name defaults to Jobname.

    ext
        Filename extension (eight-character maximum).

    kedit
        Flag to suppress nonessential commands:

        NONE - Do not suppress any commands (default).

        COMMENT - Write nonessential commands as comments (starting with !).

        REMOVE - Do not write nonessential commands or comments.

    Notes
    -----
    Writes the database command log to a named file.  The database
    command log contains all commands that were used to create the
    current database.  These commands are recorded in the database
    as they are issued, and saved in the database file (File.DB)
    whenever the database is saved.  The LGWRITE command extracts
    these commands from the database and writes them to a file.
    Nonessential commands (for listing, graphics displays, help,
    etc.) can be excluded from the file by using the Kedit field.
    The file resulting from LGWRITE can be used as command input
    to the program.  This command is most useful if the session
    log file (File.LOG), which is normally saved during an
    interactive session, has been lost or corrupted.

    This command is valid in any processor.
    """
    return self.run(f"LGWRITE,{fname},{ext},,{kedit}", **kwargs)


def starlist(self, fname="", ext="", **kwargs):
    """Displays the contents of an external, coded file.

    APDL Command: *LIST

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).  An
        unspecified directory path defaults to the working directory;
        in this case, you can use all 248 characters for the file
        name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Displays the contents of an external, coded file.  The file to be
    listed cannot be in use (open) at the time (except for the error
    file, File.ERR, which may be displayed with *LIST,ERR).

    This command is valid in any processor.
    """
    command = f"*LIST,{fname},{ext}"
    return self.run(command, **kwargs)


def rename(self, fname1="", ext1="", fname2="", ext2="", distkey="",
           **kwargs):
    """Renames a file.

    APDL Command: /RENAME

    Parameters
    ----------
    fname1
        The file to be renamed. You can also include an optional
        directory path as part of the specified file name; if not,
        the default file location is the working directory.

    ext1
        Filename extension (eight-character maximum).

    fname2
        The new name for the file. You can also include an
        optional directory path as part of the new file name; if
        not, the default is the working directory. A maximum of
        248 characters is allowed for the file name (or combined
        file name and directory path, if both are specified).

    ext2
        Filename extension (eight-character maximum).

    distkey
        Key that specifies whether the rename operation is
        performed on all processes in distributed parallel mode
        (Distributed ANSYS):

        1 (ON or YES) - The program performs the rename operation
        locally on each process.

        0 (OFF or NO) - The program performs the rename operation
        only on the master process (default).

    Notes
    -----
    Renames a file.  Ex: /RENAME,A,,,B renames file A to B in the
    same directory.  /RENAME,A,DAT,,,INP renames file A.DAT to
    A.INP. On all systems, this command will overwrite any
    existing file named B. See the Operations Guide for
    details. Only ANSYS binary files should be renamed. Use /SYS
    and system renaming commands for other files.

    In distributed parallel mode (Distributed ANSYS), only the
    master process will rename Fname1.Ext1 to Fname2.Ext2 by
    default. However, when DistKey is set to 1 (or ON or YES), the
    command is executed by all processes. In this case, Fname1 and
    Fname2 will automatically have the process rank appended to
    them. This means Fname1N.Ext1 will be renamed to Fname2N.Ext2
    by all processes, where N is the Distributed ANSYS process
    rank. For more information see Differences in General Behavior
    in the Parallel Processing Guide.

    Renaming across system partitions may be internally done by a
    copy and delete operation on some systems.

    This command is valid only at the Begin Level.
    """
    return self.run(f"/RENAME,{fname1},{ext1},,{fname2},{ext2},,{distkey}",
                    **kwargs)


def slashfdele(self, ident="", stat="", **kwargs):
    """Deletes a binary file after it is used.

    APDL Command: /FDELE

    Parameters
    ----------
    ident
        ANSYS file name identifier.  Valid identifiers are:  EMAT, ESAV,
        FULL, SUB, MODE, DSUB, USUB, OSAV, and SELD.  See the Basic
        Analysis Guide for file descriptions.

    stat
        Keep or delete key:

        KEEP - Keep this file.

        DELE - Delete (or do not write, if not necessary) this file.

    Notes
    -----
    Deletes as soon as possible (or prevents writing) a binary file created
    by the ANSYS program to save space.

    .. warning::
        Deleting files that are necessary for the next substep,
        load step, or analysis will prevent continuation of the
        run.

    This command is valid only at the Begin Level.
    """
    command = f"/FDELE,{ident},{stat}"
    return self.run(command, **kwargs)


def slashdelete(self, fname="", ext="", distkey="", **kwargs):
    """Deletes a file.

    APDL Command: /DELETE

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    distkey
        Key that specifies whether the file deletion is performed
        on all processes in distributed parallel mode (Distributed
        ANSYS):

        1 (ON or YES) - The program performs the file deletion
        locally on each process.

        0 (OFF or NO) - The program performs the file deletion
        only on the master process (default).

    Notes
    -----
    In distributed parallel mode (Distributed ANSYS), only the
    master process will delete Fname.Ext by default. However, when
    DistKey is set to 1 (or ON or YES), the command is executed by
    all processes. In this case, Fname will automatically have the
    process rank appended to it.  This means FnameN.Ext will be
    deleted by all processes, where N is the Distributed ANSYS
    process rank. For more information see Differences in General
    Behavior in the Parallel Processing Guide.
    """
    return self.run(f"/DELETE,{fname},{ext},,{distkey}", **kwargs)



def se(self, file="", toler="", **kwargs):
    """Defines a superelement.

    APDL Command: SE

    Parameters
    ----------
    file
        The name (case sensitive) of the file containing the
        original superelement matrix created by the generation
        pass (Sename.SUB).  The default is the current Jobname.

    toler
        Tolerance used to determine if use pass nodes are
        noncoincident with master nodes having the same node
        numbers.  Defaults to 0.0001.  Use pass nodes will always
        be replaced by master nodes of the same node number.
        However, if a use pass node is more than TOLER away from
        the corresponding master node, a warning is generated.

    Notes
    -----
    Defines a superelement by reading in the superelement matrices
    and master nodes from the superelement matrix file.  The
    matrix file (File.SUB) must be available from the substructure
    generation pass.  The proper element type (MATRIX50) must be
    active [TYPE] for this command.  A scratch file called
    File.SORD showing the superelement names and their
    corresponding element numbers is also written.
    """
    return self.run(f"SE,{file},,,{toler}", **kwargs)


def sedlist(self, sename="", kopt="", **kwargs):
    """Lists the DOF solution of a superelement after the use pass.

    APDL Command: SEDLIST

    Parameters
    ----------
    sename
        Name of the superelement in Jobname.DSUB to be listed.  If a
        number, it is the element number of the superelement as used in the
        use pass.  If ALL, list results for all superelements.

    kopt
        List key:

        0 - List summary data only.

        1 - List full contents.  Be aware that the listing may be extensive.

    Notes
    -----
    Lists the degree of freedom solution of a superelement after the
    substructure use pass.  Results may be listed for any superelement on
    File.DSUB.

    This command is valid in any processor.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"SEDLIST,{sename},{kopt}"
    return self.run(command, **kwargs)


def selist(self, sename="", kopt="", kint="", **kwargs):
    """Lists the contents of a superelement matrix file.

    APDL Command: SELIST

    Parameters
    ----------
    sename
        The name (case-sensitive) of the superelement matrix file created
        by the substructure generation pass (Sename.SUB).   Defaults to the
        current Jobname.  If a number, it is the element number of the
        superelement as used in the use pass.

    kopt
        List key:

        0 - List summary data only.

        1 - List contents, except load vectors and matrices.

        2 - List contents, except matrices.

        3 - List full contents.  Be aware that the listing may be extensive.

    kint
        Integer printout format key:

        OFF - Default.

        ON - Long format for large integers.

    Notes
    -----
    This command is valid in any processor.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"SELIST,{sename},{kopt},{kint}"
    return self.run(command, **kwargs)


def sesymm(self, sename="", ncomp="", inc="", file="", ext="", **kwargs):
    """Performs a symmetry operation on a superelement within the use pass.

    APDL Command: SESYMM

    Parameters
    ----------
    sename
        The name (case-sensitive) of the superelement matrix file
        created by the substructure generation pass (Sename.SUB).
        Defaults to the current Jobname.  If a number, it is the
        element number of a previously defined superelement in the
        current use pass.

    ncomp
        Symmetry key:

        X - X symmetry (default).

        Y - Y symmetry.

        Z - Z symmetry.

    inc
        Increment all nodes in the superelement by INC.

    file
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Performs a symmetry operation on a superelement within the
    substructure use pass by reversing the sign of component Ncomp
    in the global Cartesian coordinate system.  The node numbers
    are incremented by INC.  The new superelement is written to
    File.SUB in the current directory (by default).  All master
    node nodal coordinate systems must be global Cartesian (no
    rotated nodes allowed).

    The maximum number of transformations for a given superelement
    is five (including SETRAN, SESYMM, and the large rotation
    transformation if NLGEOM is ON in the use pass).

    Distributed ANSYS Restriction: This command is not supported
    in Distributed ANSYS.
    """
    command = f"SESYMM,{sename},{ncomp},{inc},{file},{ext}"
    return self.run(command, **kwargs)


def setran(self, sename="", kcnto="", inc="", file="", ext="", dx="",
           dy="", dz="", norot="", **kwargs):
    """Creates a superelement from an existing superelement.

    APDL Command: SETRAN

    Parameters
    ----------
    sename
        The name (case-sensitive) of the file containing the original
        superelement matrix created by the generation pass (Sename.SUB).
        The default is the current Jobname.  If Sename is a number, it is
        the element number of a previously defined superelement in the
        current use pass.

    kcnto
        The reference number of the coordinate system to where the
        superelement is to be transferred. The default is the global
        Cartesian system.  Transfer occurs from the active coordinate
        system.

    inc
        The node offset.  The default is zero.  All new element node
        numbers are offset from those on the original by INC.

    file
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    dx, dy, dz
        Node location increments in the global Cartesian coordinate system.
        Defaults to zero.

    norot
        Node rotation key:

        0 - The nodal coordinate systems of the transferred superelement rotate into the
            KCNTO system. (That is, the nodal coordinate systems rotate
            with the superelement.) The superelement matrices remain
            unchanged. This value is the default.

        1 - The nodal coordinate systems do not rotate. (That is, they remain fixed in
            their original global orientation.)  The superelement matrices
            and load vectors are modified if any rotations occur.

    Notes
    -----
    The SETRAN command creates a superelement from an existing superelement
    and writes the new element to a file.  You can then issue an SE command
    to read the new element (during the use pass).

    You can create a superelement from an original by:

    Transferring the original's geometry from the active coordinate system
    into another coordinate system (KCNTO)

    Offsetting its geometry in the global Cartesian coordinate system (DX,
    DY, and DZ )

    Offsetting its node numbers (INC).

    A combination of methods is valid. If you specify both the geometry
    transfer and the geometry offset, the transfer occurs first.

    If you specify rotation of the transferred superelement's nodal
    coordinate systems into the KCNTO system (NOROT = 0), the rotated nodes
    cannot be coupled via the CP command; in this case, issue the CE
    command instead. If you specify no rotation of the nodal coordinate
    systems (NOROT = 1) for models with displacement degrees of freedom,
    and KCNTO is not the active system, the superelement Sename must have
    six MDOF at each node that has MDOF; therefore, only elements with all
    six structural DOFs are valid in such cases.

    There is no limit to the number of copies that can be made of a
    superelement, provided the copies are all generated from the same
    original superelement. However, nested copies are limited to five. In
    other words, the total number of different Sename usages on the SETRAN
    and SESYMM commands is limited to five.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"SETRAN,{sename},{kcnto},{inc},{file},{ext},,{dx},{dy},{dz},{norot}"
    return self.run(command, **kwargs)



def stef(self, value="", **kwargs):
    """Specifies Stefan-Boltzmann radiation constant.

    APDL Command: STEF

    Parameters
    ----------
    value
        Stefan-Boltzmann constant (defaults to 0.119E-10 Btu/hr/in2/ °R4).

    Notes
    -----
    You can use this command in the general preprocessor (PREP7) and in the
    Solution processor to specify the Stefan-Boltzmann constant in analyses
    using the radiation matrix method or the radiosity solver to model
    radiation.
    """
    command = f"STEF,{value}"
    return self.run(command, **kwargs)


def toffst(self, value="", **kwargs):
    """Specifies the temperature offset from absolute zero to zero.

    APDL Command: TOFFST

    Parameters
    ----------
    value
        Degrees between absolute zero and zero of temperature system used
        (should be positive).

    Notes
    -----
    Specifies the difference (in degrees) between absolute zero and the
    zero of the temperature system used.  Absolute temperature values are
    required in evaluating certain expressions, such as for creep,
    swelling, radiation heat transfer, MASS71, etc.  (The offset
    temperature is not used in evaluating emissivity.) Examples are 460°
    for the Fahrenheit system and 273° for the Celsius system.  The offset
    temperature is internally included in the element calculations and does
    not affect the temperature input or output.  If used in SOLUTION, this
    command is valid only within the first load step.

    This command is also valid in PREP7.
    """
    command = f"TOFFST,{value}"
    return self.run(command, **kwargs)



def bfa(self, area="", lab="", val1="", val2="", val3="", val4="",
        **kwargs):
    """Defines a body force load on an area.

    APDL Command: BFA

    Parameters
    ----------
    area
        Area to which body load applies.  If ALL, apply to all selected
        areas [ASEL]. A component name may also be substituted for Area.

    lab
        Valid body load label. Load labels are listed under "Body Loads" in
        the input table for each element type in the Element Reference.

    val1, val2, val3
        Value associated with the Lab item or a table name for specifying
        tabular boundary conditions. Use only VAL1 for TEMP, FLUE, HGEN,
        CHRGD. For Lab = JS in magnetics, use VAL1, VAL2, and VAL3 for the
        X, Y, and Z components. For acoustics, if Lab = JS, use VAL1 for
        mass source in a harmonic analysis or mass source rate in a
        transient analysis, and ignore VAL2 and VAL3. For Lab = VLTG, VAL1
        is the voltage drop and VAL2 is the phase angle. If Lab = IMPD,
        VAL1 is the resistance and VAL2 is the reactance in ohms/square.
        When specifying a table name, you must enclose the table name in
        percent signs (%), e.g., BFA,Area,Lab,%tabname%. Use the *DIM
        command to define a table.

    val4
        If Lab = JS, VAL4 is the phase angle in degrees.

    Notes
    -----
    Defines a body force load (such as temperature in a structural
    analysis, heat generation rate in a thermal analysis, etc.) on an area.
    Body loads may be transferred from areas to area elements (or to nodes
    if area elements do not exist) with the BFTRAN or SBCTRAN commands.
    Body loads default to the value specified on the BFUNIF command, if it
    was previously specified.

    You can specify a table name only when using temperature (TEMP) and
    heat generation rate (HGEN) body load labels.

    Body loads specified by the BFA command can conflict with other
    specified body loads.  See Resolution of Conflicting Body Load
    Specifications in the Basic Analysis Guide for details.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFA,{area},{lab},{val1},{val2},{val3},{val4}"
    return self.run(command, **kwargs)


def bfadele(self, area="", lab="", **kwargs):
    """Deletes body force loads on an area.

    APDL Command: BFADELE

    Parameters
    ----------
    area
        Area at which body load is to be deleted.  If ALL, delete for all
        selected areas [ASEL]. A component name may also be substituted for
        AREA.

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFA command for
        labels.

    Notes
    -----
    Deletes body force loads (and all corresponding finite element loads)
    for a specified area and label.  Body loads may be defined on an area
    with the BFA command.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFADELE,{area},{lab}"
    return self.run(command, **kwargs)


def bfalist(self, area="", lab="", **kwargs):
    """Lists the body force loads on an area.

    APDL Command: BFALIST

    Parameters
    ----------
    area
        Area at which body load is to be listed.  If ALL (or blank), list
        for all selected areas [ASEL].  If AREA = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for AREA.

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFA command for
        labels.

    Notes
    -----
    Lists the body force loads for the specified area and label.  Body
    loads may be defined on an area with the BFA command.

    This command is valid in any processor.
    """
    command = f"BFALIST,{area},{lab}"
    return self.run(command, **kwargs)


def bfk(self, kpoi="", lab="", val1="", val2="", val3="", phase="",
        **kwargs):
    """Defines a body force load at a keypoint.

    APDL Command: BFK

    Parameters
    ----------
    kpoi
        Keypoint to which body load applies.  If ALL, apply to all selected
        keypoints [KSEL].  A component name may also be substituted for
        Kpoi.

    lab
        Valid body load label. Load labels are listed under "Body Loads" in
        the input table for each element type in the Element Reference.

    val1, val2, val3
        Value associated with the Lab item or a table name for specifying
        tabular boundary conditions.  Use only VAL1 for TEMP, FLUE, HGEN,
        MVDI and CHRGD.  For magnetics, use VAL1, VAL2, and VAL3 for the X,
        Y, and Z components of JS . For acoustics, if Lab = JS,  use VAL1
        for mass source in a harmonic analysis or mass source rate in a
        transient analysis, and ignoreVAL2 and VAL3. When specifying a
        table name, you must enclose the table name in percent signs (%),
        e.g., BFK,Kpoi,Lab,%tabname%.  Use the *DIM command to define a
        table.

    phase
        Phase angle in degrees associated with the JS label.

    Notes
    -----
    Defines a body force load (such as temperature in a structural
    analysis, heat generation rate in a thermal analysis, etc.) at a
    keypoint.  Body loads may be transferred from keypoints to nodes with
    the BFTRAN or SBCTRAN commands.  Interpolation will be used to apply
    loads to the nodes on the lines between keypoints.  All keypoints on a
    given area (or volume) must have the same BFK specification, with the
    same values, for the loads to be transferred to interior nodes in the
    area (or volume).  If only one keypoint on a line has a BFK
    specification, the other keypoint defaults to the value specified on
    the BFUNIF command.

    You can specify a table name only when using temperature (TEMP) and
    heat generation rate (HGEN) body load labels.

    Body loads specified by the BFK command can conflict with other
    specified body loads.  See Resolution of Conflicting Body Load
    Specifications in the Basic Analysis Guide for details.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFK,{kpoi},{lab},{val1},{val2},{val3},{phase}"
    return self.run(command, **kwargs)


def bfkdele(self, kpoi="", lab="", **kwargs):
    """Deletes body force loads at a keypoint.

    APDL Command: BFKDELE

    Parameters
    ----------
    kpoi
        Keypoint at which body load is to be deleted.  If ALL, delete for
        all selected keypoints [KSEL]. A component name may also be
        substituted for KPOI.

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFK command for
        labels.

    Notes
    -----
    Deletes body force loads (and all corresponding finite element loads)
    for a specified keypoint and label.  Body loads may be defined at a
    keypoint with the BFK command.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFKDELE,{kpoi},{lab}"
    return self.run(command, **kwargs)


def bfklist(self, kpoi="", lab="", **kwargs):
    """Lists the body force loads at keypoints.

    APDL Command: BFKLIST

    Parameters
    ----------
    kpoi
        Keypoint at which body load is to be listed.  If ALL (or blank),
        list for all selected keypoints [KSEL].  If KPOI = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for KPOI

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFK command for
        labels.

    Notes
    -----
    Lists the body force loads for the specified keypoint and label.
    Keypoint body loads may be defined with the BFK command.

    This command is valid in any processor.
    """
    command = f"BFKLIST,{kpoi},{lab}"
    return self.run(command, **kwargs)


def bfl(self, line="", lab="", val1="", val2="", val3="", val4="",
        **kwargs):
    """Defines a body force load on a line.

    APDL Command: BFL

    Parameters
    ----------
    line
        Line to which body load applies.  If ALL, apply to all selected
        lines [LSEL]. A component name may also be substituted for Line.

    lab
        Valid body load label. Load labels are listed under "Body loads" in
        the input table for each element type in the Element Reference.

    val1, val2, val3
        Value associated with the Lab item or a table name for specifying
        tabular boundary conditions.  Use only VAL1 for TEMP, FLUE, HGEN,
        and CHRGD. For acoustics, if Lab = JS,  use VAL1 for mass source in
        a harmonic analysis or mass source rate in a transient analysis,
        and ignoreVAL2 and VAL3. When specifying a table name, you must
        enclose the table name in percent signs (%), e.g.,
        BFL,Line,Lab,%tabname%.  Use the *DIM command to define a table.

    val4
        If Lab = JS, VAL4 is the phase angle in degrees.

    Notes
    -----
    Defines a body force load (such as temperature in a structural
    analysis, heat generation rate in a thermal analysis, etc.) on a line.
    Body loads may be transferred from lines to line elements (or to nodes
    if line elements do not exist) with the BFTRAN or SBCTRAN commands.

    You can specify a table name only when using temperature (TEMP) and
    heat generation rate (HGEN) body load labels.

    Body loads specified by the BFL command can conflict with other
    specified body loads.  See Resolution of Conflicting Body Load
    Specifications in the Basic Analysis Guide for details.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFL,{line},{lab},{val1},{val2},{val3},{val4}"
    return self.run(command, **kwargs)


def bfldele(self, line="", lab="", **kwargs):
    """Deletes body force loads on a line.

    APDL Command: BFLDELE

    Parameters
    ----------
    line
        Line at which body load is to be deleted.  If ALL, delete for all
        selected lines [LSEL].  A component name may also be substituted
        for LINE.

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFL command for
        labels.

    Notes
    -----
    Deletes body force loads (and all corresponding finite element loads)
    for a specified line and label.  Body loads may be defined on a line
    with the BFL command.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFLDELE,{line},{lab}"
    return self.run(command, **kwargs)


def bfllist(self, line="", lab="", **kwargs):
    """Lists the body force loads on a line.

    APDL Command: BFLLIST

    Parameters
    ----------
    line
        Line at which body load is to be listed.  If ALL (or blank), list
        for all selected lines [LSEL].  If LINE = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for LINE.

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFL command for
        labels.

    Notes
    -----
    Lists the body force loads for the specified line and label.  Body
    loads may be defined on a line with the BFL command.

    This command is valid in any processor.
    """
    command = f"BFLLIST,{line},{lab}"
    return self.run(command, **kwargs)


def bftran(self, **kwargs):
    """Transfers solid model body force loads to the finite element model.

    APDL Command: BFTRAN

    Notes
    -----
    Body loads are transferred from selected keypoints and lines to
    selected nodes and from selected areas and volumes to selected
    elements.  The BFTRAN operation is also done if the SBCTRAN command is
    either explicitly issued or automatically issued upon initiation of the
    solution calculations [SOLVE].

    This command is also valid in PREP7.
    """
    command = f"BFTRAN,"
    return self.run(command, **kwargs)


def bfv(self, volu="", lab="", val1="", val2="", val3="", phase="",
        **kwargs):
    """Defines a body force load on a volume.

    APDL Command: BFV

    Parameters
    ----------
    volu
        Volume to which body load applies.  If ALL, apply to all selected
        volumes [VSEL]. A component name may also be substituted for Volu.

    lab
        Valid body load label. Load labels are listed under "Body Loads" in
        the input table for each element type in the Element Reference.

    val1, val2, val3
        Value associated with the Lab item or a table name for specifying
        tabular boundary conditions.  Use only VAL1 for TEMP, FLUE, HGEN,
        and CHRGD.  Use VAL1, VAL2, and VAL3 for the X, Y, and Z components
        of JS.  For Lab = JS in magnetics, use VAL1, VAL2, and VAL3 for the
        X, Y, and Z components. For acoustics, if Lab = JS,  use VAL1 for
        mass source in a harmonic analysis or mass source rate in a
        transient analysis, and ignoreVAL2 and VAL3. For Lab = VLTG, VAL1
        is the voltage drop and VAL2 is the phase angle.  When specifying a
        table name, you must enclose the table name in percent signs (%),
        e.g., BFV,Volu,Lab,%tabname%.  Use the *DIM command to define a
        table.

    phase
        Phase angle in degrees associated with the JS label.

    Notes
    -----
    Defines a body force load (such as temperature in a structural
    analysis, heat generation rate in a thermal analysis, etc.) on a
    volume.  Body loads may be transferred from volumes to volume elements
    (or to nodes if volume elements do not exist) with the BFTRAN or
    SBCTRAN commands.  Body loads default to the value specified on the
    BFUNIF command, if it was previously specified.

    You can specify a table name only when using temperature (TEMP) and
    heat generation rate (HGEN) body load labels.

    Body loads specified by the BFV command can conflict with other
    specified body loads.  See Resolution of Conflicting Body Load
    Specifications in the Basic Analysis Guide for details.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFV,{volu},{lab},{val1},{val2},{val3},{phase}"
    return self.run(command, **kwargs)


def bfvdele(self, volu="", lab="", **kwargs):
    """Deletes body force loads on a volume.

    APDL Command: BFVDELE

    Parameters
    ----------
    volu
        Volume at which body load is to be deleted.  If ALL, delete for all
        selected volumes [VSEL]. A component name may also be substituted
        for VOLU.

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFV command for
        labels.

    Notes
    -----
    Deletes body force loads (and all corresponding finite element loads)
    for a specified volume and label.  Body loads may be defined on a
    volume with the BFV command.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFVDELE,{volu},{lab}"
    return self.run(command, **kwargs)


def bfvlist(self, volu="", lab="", **kwargs):
    """Lists the body force loads on a volume.

    APDL Command: BFVLIST

    Parameters
    ----------
    volu
        Volume at which body load is to be listed.  If ALL (or blank), list
        for all selected volumes [VSEL].  If VOLU = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for VOLU.

    lab
        Valid body load label. If ALL, use all appropriate labels. Load
        labels are listed under "Body Loads" in the input table for each
        element type in the Element Reference. See the BFV command for
        labels.

    Notes
    -----
    Lists the body force loads for the specified volume and label.  Body
    loads may be defined on a volume with the BFV command.

    This command is valid in any processor.
    """
    command = f"BFVLIST,{volu},{lab}"
    return self.run(command, **kwargs)



def lcabs(self, lcno="", kabs="", **kwargs):
    """Specifies absolute values for load case operations.

    APDL Command: LCABS

    Parameters
    ----------
    lcno
        Load case pointer number.  If ALL, apply to all selected load cases
        [LCSEL].

    kabs
        Absolute value key:

        0 - Use algebraic values of load case LCNO in operations.

        1 - Use absolute values of load case LCNO in operations.

    Notes
    -----
    Causes absolute values to be used in the load case operations [LCASE or
    LCOPER].  Absolute values are taken prior to assigning a load case
    factor [LCFACT] and are applied only to defined load cases [LCDEF].
    """
    command = f"LCABS,{lcno},{kabs}"
    return self.run(command, **kwargs)


def lcase(self, lcno="", **kwargs):
    """Reads a load case into the database.

    APDL Command: LCASE

    Parameters
    ----------
    lcno
        Load case pointer number [LCDEF,STAT].  Defaults to 1.

    Notes
    -----
    Reads a load case into the database.  Load cases are created as
    described on the LCDEF or LCWRITE commands.  The results portion of the
    database and the applied forces and displacements are cleared before
    reading the data in.  Absolute values [LCABS] and scale factors
    [LCFACT] can be applied during the read operation.
    """
    command = f"LCASE,{lcno}"
    return self.run(command, **kwargs)


def lcdef(self, lcno="", lstep="", sbstep="", kimg="", **kwargs):
    """Creates a load case from a set of results on a results file.

    APDL Command: LCDEF

    Parameters
    ----------
    lcno
        Arbitrary pointer number (1-99) to be assigned to the load case
        specified by LSTEP, SBSTEP and by the FILE command.  Defaults to 1
        + previous value.

    lstep
        Load step number to be defined as the load case.  Defaults to one.

    sbstep
        Substep number.  Defaults to the last substep of the load step.

    kimg
        Used only with results from complex analyses:

        0 - Use real part of complex solution

        1 - Use imaginary part.

    Notes
    -----
    Creates a load case by establishing a pointer to a set of results on a
    results file (written during the ANSYS solution phase).  This pointer
    (LCNO) can then be used on the LCASE or LCOPER commands to read the
    load case data into the database.

    Issue LCDEF,ERASE to delete all load case pointers (and all load case
    files, if any).  Issue LCDEF,LCNO,ERASE to delete only the specific
    load case pointer LCNO (and its file, if any).  With the ERASE options,
    all pointers are deleted; however only files with the default extension
    [LCWRITE] are deleted.  Issue LCDEF,STAT for status of all selected
    load cases [LCSEL], or LCDEF,STAT,ALL for status of all load cases.
    The STAT command may be used to list all load cases.  See also LCFILE
    to establish a pointer to a set of results on a load case file (written
    by LCWRITE). Harmonic element data read from a result file load case is
    stored at the zero-degree position.
    """
    command = f"LCDEF,{lcno},{lstep},{sbstep},{kimg}"
    return self.run(command, **kwargs)


def lcfact(self, lcno="", fact="", **kwargs):
    """Defines scale factors for load case operations.

    APDL Command: LCFACT

    Parameters
    ----------
    lcno
        Load case pointer number.  If ALL, apply to all selected load cases
        [LCSEL].

    fact
        Scale factor applied to load case LCNO.  Blank defaults to 1.0.

    Notes
    -----
    Defines scale factors to be used in the load case operations [LCASE or
    LCOPER].  Scale factors are applied after an absolute value operation
    [LCABS] and are applied only to defined load cases [LCDEF].
    """
    command = f"LCFACT,{lcno},{fact}"
    return self.run(command, **kwargs)


def lcfile(self, lcno="", fname="", ext="", **kwargs):
    """Creates a load case from an existing load case file.

    APDL Command: LCFILE

    Parameters
    ----------
    lcno
        Arbitrary (1-99) pointer number assigned to this load case.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Creates a load case by establishing a pointer to an existing load case
    file [LCWRITE].  This pointer (LCNO) can then be used on the LCASE or
    LCOPER commands to read the load case data into the database.  This
    command is typically used to reestablish load case pointers in a new
    ANSYS session (pointers are not saved on the database file), or when
    more than one pointer to a single load case is desired.  See the LCDEF
    command for status and erase operations.  See also LCDEF to establish a
    pointer to a set of results on a results file (written during the ANSYS
    solution phase).
    """
    command = f"LCFILE,{lcno},{fname},{ext}"
    return self.run(command, **kwargs)


def lcoper(self, oper="", lcase1="", oper2="", lcase2="", **kwargs):
    """Performs load case operations.

    APDL Command: LCOPER

    Parameters
    ----------
    oper
        Valid operations are:

        ZERO - Zero results portion of database (LCASE1 ignored).

        SQUA - Square database values (LCASE1 ignored).

        SQRT - Square root of database (absolute) values (LCASE1 ignored).

        LPRIN - Recalculate line element principal stresses (LCASE1 ignored).  Stresses are as
                shown for the NMISC items of the ETABLE command for the
                specific line element type.

        ADD - Add LCASE1 to database values.

        SUB - Subtract LCASE1 from database values.

        SRSS - Square root of the sum of the squares of database and LCASE1.

        MIN - Compare and save in database the algebraic minimum of database and LCASE1.

        MAX - Compare and save in database the algebraic maximum of database and LCASE1.

        ABMN - Compare and save in database the absolute minimum of database and LCASE1 (based
               on magnitudes, then apply the corresponding sign).

        ABMX - Compare and save in database the absolute maximum of database and LCASE1 (based
               on magnitudes, then apply the corresponding sign).

    lcase1
        First load case in the operation (if any).  See LCNO of the LCDEF
        command. If ALL, repeat operations using all selected load cases
        [LCSEL].

    oper2
        Valid operations are:

        MULT - Multiplication: LCASE1*LCASE2

        CPXMAX - This option does a phase angle sweep to calculate the maximum of derived
                 stresses and equivalent strain for a complex solution
                 where LCASE1 is the real part and LCASE2 is the imaginary
                 part. The Oper field is not applicable with  this option.
                 Also, the LCABS and SUMTYPE commands have no effect on
                 this option. The value of S3 will be a minimum.   This
                 option does not apply to derived displacement amplitude
                 (USUM). Load case writing (LCWRITE) is not supported. See
                 POST1 and POST26 – Complex Results Postprocessing in the
                 Mechanical APDL Theory Reference for more information.

    lcase2
        Second load case.  Used only with Oper2 operations.

    Notes
    -----
    LCOPER operates on the database and one or two load cases according to:

    Database = Database Oper (LCASE1 Oper2 LCASE2)

    where operations Oper and Oper2 are as described above.  Absolute
    values and scale factors may be applied to the load cases before the
    operations [LCABS, LCFACT].  If LCASE1 is not specified, only operation
    Oper is performed on the current database.  If LCASE2 is specified,
    operation Oper2 will be performed before operation Oper.  If LCASE2 is
    not specified, operation Oper2 is ignored.  Solution items not
    contained [OUTRES] in either the database or the applicable load cases
    will result in a null item during a load case operation.  Harmonic
    element data read from a result file load case are processed at zero
    degrees.  All load case combinations are performed in the solution
    coordinate system, and the data resulting from load case combinations
    are stored in the solution coordinate system.  The resultant data are
    then transformed to the active results coordinate system [RSYS] when
    listed or displayed. Except in the cases of Oper = LPRIN, ADD, or SUB,
    you must use RSYS,SOLU to list or display, and in the case of layered
    elements, the layer (LAYER) must also be specified.

    Use the FORCE command prior to any combination operation to correctly
    combine the requested force type.

    If Oper2=CPXMAX, the derived stresses and strain calculation do not
    apply to line elements.
    """
    command = f"LCOPER,{oper},{lcase1},{oper2},{lcase2}"
    return self.run(command, **kwargs)


def lcsel(self, type_="", lcmin="", lcmax="", lcinc="", **kwargs):
    """Selects a subset of load cases.

    APDL Command: LCSEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        S - Select a new set.

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

        ALL - Restore the full set.

        NONE - Unselect the full set.

        INVE - Invert the current set (selected becomes unselected and vice versa).

        STAT - Display the current select status.

    lcmin
        Minimum value of load case pointer range.

    lcmax
        Maximum value of load case pointer range.  LCMAX defaults to LCMIN.

    lcinc
        Value increment within range.  Defaults to 1.  LCINC cannot be
        negative.

    Notes
    -----
    Selects a subset of load cases for other operations.  For example, to
    select a new set of load cases based on load cases 1 through 7, use
    LCSEL,S,1,7.  The subset is used when the ALL label is entered (or
    implied) on other commands, such as LCFACT, LCABS, LCOPER, etc.  Load
    cases are flagged as selected and unselected; no load case pointers
    [LCDEF, LCWRITE, LCFILE] are actually deleted from the database.
    """
    command = f"LCSEL,{type_},{lcmin},{lcmax},{lcinc}"
    return self.run(command, **kwargs)


def lcsum(self, lab="", **kwargs):
    """Specifies whether to process non-summable items in load case

    APDL Command: LCSUM
    operations.

    Parameters
    ----------
    lab
        Combination option

        (blank) - Only combine summable items [default].

        ALL - Combine all items including non summable items.

    Notes
    -----
    Allows non-summable items (e.g. plastic strains) to be included in load
    combinations.  Issue LCSUM,ALL before the first load case operation
    (LCXX command).  May also be used to include nonsummable items in the
    appending of a results file (RAPPND command).
    """
    command = f"LCSUM,{lab}"
    return self.run(command, **kwargs)


def lcwrite(self, lcno="", fname="", ext="", **kwargs):
    """Creates a load case by writing results to a load case file.

    APDL Command: LCWRITE

    Parameters
    ----------
    lcno
        Arbitrary pointer number (1-99) to be assigned to this load case.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Creates a load case by writing the results data in the database to a
    load case file.  The database remains unchanged by this operation.  A
    pointer is also established to the written set of results on the load
    case file.  This pointer (LCNO) can then be used on the LCASE or LCOPER
    commands to read the load case data into the database.  By default,
    only summable results data (such as displacements, stresses, elastic
    strains) and constant results data (such as volume) are written to the
    load case file unless requested (LCSUM command).  Non-summable results
    data (such as plastic strains, strain energy), boundary conditions, and
    nodal loads are not written to the load case file.  The load case file
    may be named by default or by a user name.  Rewriting to the same file
    overwrites the previous data.  See the LCDEF command for status and
    erase operations.
    """
    command = f"LCWRITE,{lcno},{fname},{ext}"
    return self.run(command, **kwargs)


def lczero(self, **kwargs):
    """Zeroes the results portion of the database.

    APDL Command: LCZERO

    Notes
    -----
    Often used before the LCOPER command.  Same as LCOPER,ZERO.
    """
    command = f"LCZERO,"
    return self.run(command, **kwargs)


def rappnd(self, lstep="", time="", **kwargs):
    """Appends results data from the database to the results file.

    APDL Command: RAPPND

    Parameters
    ----------
    lstep
        Load step number to be assigned to the results data set.  If it is
        the same as an existing load step number on the results file, the
        appended load step will be inaccessible.  Defaults to 1.

    time
        Time value to be assigned to the results data set.  Defaults to
        0.0.  A time value greater than the last load step should be used.

    Notes
    -----
    This command is typically used to append the results from a load case
    combination to the results file.  See the LCWRITE command to create a
    separate load case file.  Only summable and constant data are written
    to the results file by default; non-summable data are not written
    unless requested (LCSUM command). RAPPND should not be used to append
    results from a harmonic analysis.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"RAPPND,{lstep},{time}"
    return self.run(command, **kwargs)



def d(self, node="", lab="", value="", value2="", nend="", ninc="",
      lab2="", lab3="", lab4="", lab5="", lab6="", **kwargs):
    """Defines degree-of-freedom constraints at nodes.

    APDL Command: D

    Parameters
    ----------
    node
        Node at which constraint is to be specified. If ALL, NEND and NINC
        are ignored and constraints are applied to all selected nodes
        (NSEL). If Node = P, graphical picking is enabled and all remaining
        command fields are ignored (valid only in the GUI).  A component
        name may also be substituted for Node.

    lab
        Valid degree-of-freedom label.  If ALL, use all appropriate labels.

    value
        Degree-of-freedom value or table name reference for tabular
        boundary conditions.  To specify a table, enclose the table name in
        percent (%) signs (for example, D,Node,TEMP,%tabname%). Use the
        *DIM command to define a table.

    value2
        Second degree-of-freedom value (if any).  If the analysis type and
        the degree of freedom allow a complex input, Value (above) is the
        real component and VALUE2 is the imaginary component.

    nend, ninc
        Specifies the same values of constraint at the range of nodes from
        Node to NEND (defaults to Node), in steps of NINC (defaults to 1).

    lab2, lab3, lab4, lab5, lab6
        Additional degree-of-freedom labels. The same values are applied to
        the nodes for these labels.

    Notes
    -----
    The available degrees of freedom per node are listed under "Degrees of
    Freedom" in the input table for each element type in the Element
    Reference.  Degrees of freedom are defined in the nodal coordinate
    system.  The positive directions of structural translations and
    rotations are along and about the positive nodal axes directions.
    Structural rotations should be input in radians.  The node and the
    degree-of-freedom label must be selected (NSEL, DOFSEL).

    In a structural analysis, you can apply only one displacement,
    velocity, or acceleration load at any degree of freedom. If multiple
    loads are specified, the last applied load overrides the previous ones.
    For example, the following commands apply loads to node 100:

    In this case, the velocity load (VELX) applied in the last command will
    override the displacement load (UX).

    For elements used in static and low frequency electromagnetic analysis
    (SOLID236 and SOLID237), the AZ degree of freedom is not a z-component
    of a vector potential, but rather the flux contribution on the element
    edge.  To specify a flux-parallel condition, set AZ = 0.  For more
    information, see 3-D Magnetostatics and Fundamentals of Edge-based
    Analysis in the Low-Frequency Electromagnetic Analysis Guide.

    In an explicit dynamic analysis, the D command can only be used to fix
    nodes in the model. The degree-of-freedom value must be zero; no other
    values are valid. Use the EDLOAD command to apply a nonzero
    displacement in an explicit dynamic analysis.

    For ELBOW290 cross-section degrees of freedom (Lab = SE, SO, SW, SRA,
    SRT, or SECT), the D command can only specify fixed constraints. The
    degree-of-freedom value must be zero; no other values are valid.

    For hydrostatic fluid elements (HSFLD241 and HSFLD242), the HDSP
    degree-of-freedom constraint at the pressure node prescribes the
    pressure value for all the fluid elements sharing the pressure node.

    Tabular boundary conditions (VALUE = %tabname%) are available only for
    the following degree-of-freedom labels: Electric (VOLT), structural
    (UX, UY, UZ, ROTX, ROTY, ROTZ, and velocity and acceleration loads
    VELX, VELY, VELZ, OMGX, OMGY, OMGZ, ACCX, ACCY, ACCZ, DMGX, DMGY,
    DMGZ), acoustic (PRES, UX, UY, UZ,), and temperature (TEMP, TBOT, TE2,
    TE3, . . ., TTOP). All labels are valid only in static (ANTYPE,STATIC)
    and full transient (ANTYPE,TRANS) analyses.

    %_FIX% is an ANSYS reserved table name. When VALUE is set to %_FIX%,
    ANSYS will prescribe the degree of freedom to the "current" relative
    displacement value. This option is only valid for the following labels:
    UX, UY, UZ, ROTX, ROTY, ROTZ. Alternatively, functions UX(), UY(), etc.
    may be used (see *GET for a complete list of available functions). In
    most cases, %_FIX% usage is efficient and recommended for all
    structural degrees of freedom.

    When Value = SUPPORT, specify only the minimum number of displacement
    constraints necessary to prevent rigid body motion: three constraints
    (or fewer, depending on the element type) for 2-D models and six (or
    fewer) for 3-D models.

    If constraints and initial conditions (IC) are applied at the same
    node, the constraint specification overrides. This combination is
    useful when a constraint degree-of-freedom value needs to start with a
    nonzero value at time = 0.0. For example, if the constraint degree-of-
    freedom value is prescribed to be a cosine function, then specifying an
    initial condition for the same node and degree of freedom ensures that
    the initial value for the constraint degree of freedom at time = 0.0 is
    same as the cosine function evaluated at time = 0.0. If initial
    conditions are not specified, the constraint degree-of-freedom value
    ramps from zero in the first substep of the first loadstep.

    If more than one rotational degrees of freedom are constrained with
    non-zero rotations (ROTX, ROTY, ROTZ), rotational velocities (OMGX,
    OMGY, OMGZ), or rotational accelerations (DMGX, DMGY, DMGZ), then the
    rotation of the constrained node from its initial configuration to its
    final configuration depends on the combination and the sequence in
    which the constraints are applied. See Rotations in a Large-Deflection
    Analysis in Structural Analysis Guide.

    This command is also valid in PREP7.
    """
    command = f"D,{node},{lab},{value},{value2},{nend},{ninc},{lab2},{lab3},{lab4},{lab5},{lab6}"
    return self.run(command, **kwargs)


def dcum(self, oper="", rfact="", ifact="", tbase="", **kwargs):
    """Specifies that DOF constraint values are to be accumulated.

    APDL Command: DCUM

    Parameters
    ----------
    oper
        Accumulation key:

        REPL - Subsequent values replace the previous values (default).

        ADD - Subsequent values are added to the previous values.

        IGNO - Subsequent values are ignored.

    rfact
        Scale factor for the real component.  Zero (or blank) defaults to
        1.0.  Use a small number for a zero scale factor.

    ifact
        Scale factor for the imaginary component.  Zero (or blank) defaults
        to 1.0.  Use a small number for a zero scale factor.

    tbase
        Base temperature for temperature difference.  Used only with
        temperature degree of freedom.  Scale factor is applied to the
        temperature difference (T-TBASE) and then added to TBASE.  T is the
        current temperature.

    Notes
    -----
    Allows repeated degree of freedom constraint values (displacement,
    temperature, etc.)  to be replaced, added, or ignored.  Operations
    apply to the selected nodes [NSEL] and the selected degree of freedom
    labels [DOFSEL]. This command also operates on velocity and
    acceleration loads applied in a structural analysis.

    The operations occur when the next degree of freedom constraints are
    defined.  For example, issuing the command D,1,UX,.025 after a previous
    D,1,UX,.020 causes the new value of the displacement on node 1 in the
    x-direction to be 0.045 with the add operation, 0.025 with the replace
    operation, or 0.020 with the ignore operation.  Scale factors are also
    available to multiply the next value before the add or replace
    operation.  A scale factor of 2.0 with the previous "add" example
    results in a displacement of 0.070.  Scale factors are applied even if
    no previous values exist.  Issue DCUM,STAT to show the current label,
    operation, and scale factors.  Solid model boundary conditions are not
    affected by this command, but boundary conditions on the FE model are
    affected.

    Note:: : FE boundary conditions may still be overwritten by existing
    solid model boundary conditions if a subsequent boundary condition
    transfer occurs.

    DCUM does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"DCUM,{oper},{rfact},{ifact},{tbase}"
    return self.run(command, **kwargs)


def ddele(self, node="", lab="", nend="", ninc="", rkey="", **kwargs):
    """Deletes degree-of-freedom constraints.

    APDL Command: DDELE

    Parameters
    ----------
    node
        Node for which constraint is to be deleted.  If ALL, NEND and NINC
        are ignored and constraints for all selected nodes [NSEL] are
        deleted.  If NODE = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).  A
        component name may also be substituted for NODE.

    lab
        Valid degree of freedom label.  If ALL, use all selected labels
        [DOFSEL].  Structural labels:  UX, UY, or UZ (displacements); ROTX,
        ROTY, or ROTZ (rotations); WARP (warping).  Thermal labels: TEMP,
        TBOT, TE2, TE3, . . ., TTOP (temperature).  Acoustic labels:  PRES
        (pressure); UX, UY, or UZ (displacements for FSI coupled elements).
        Electric label:  VOLT (voltage).  Magnetic labels:  MAG (scalar
        magnetic potential); AX, AY, or AZ (vector magnetic potentials).
        Diffusion label: CONC (concentration).

    nend, ninc
        Delete constraints from NODE to NEND (defaults to NODE) in steps of
        NINC (defaults to 1).

    rkey
        Ramping key:

        OFF  - Loads are step-removed (default).

        ON or FORCE  - Forces on the specified degrees of freedom (Lab) are ramped during the next
                       load step. The forces are ramped from the reaction
                       forces of the previous load step, regardless of
                       whether or not a constraint was present. If the
                       specified node(s) and degree(s) of freedom has a
                       force value currently defined, the force is ramped
                       from the reaction force value to the currently
                       applied force value. If no force is currently
                       applied, the force is ramped from the reaction force
                       value to zero.

    Notes
    -----
    Deleting a constraint is not the same as setting it to zero (which
    "fixes" the degree of freedom to a zero value).  Deleting a constraint
    has the same effect as deactivating, releasing, or setting the
    constraint "free."  The node and the degree of freedom label must be
    selected [NSEL, DOFSEL].

    This command is also valid in PREP7.
    """
    command = f"DDELE,{node},{lab},{nend},{ninc},{rkey}"
    return self.run(command, **kwargs)


def dflx(self, node="", bx="", by="", bz="", bx2="", by2="", bz2="",
         **kwargs):
    """Imposes a uniform magnetic flux B on an edge-element electromagnetic

    APDL Command: DFLX
    model.

    Parameters
    ----------
    node
        Nodes at which the edge-flux (AZ) constraints corresponding to the
        uniform magnetic flux are to be specified.  Valid options are ALL
        (default) or Component Name. If ALL, constraints are applied to all
        selected nodes (NSEL).

    bx, by, bz
        Real components of magnetic flux B.

    bx2, by2, bz2
        Imaginary components of magnetic flux B.

    Notes
    -----
    The DFLX command sets the constraints on the edge-flux (AZ) degrees of
    freedom to produce a uniform magnetic flux B in an edge-based
    electromagnetic analysis using elements SOLID236 and SOLID237. The
    command ignores the corner nodes of the elements (even if they were
    selected) and imposes the AZ-constraints on the mid-side nodes only.
    The AZ-constraints are imposed in the active Cartesian coordinate
    system. A non-Cartesian coordinate system will be ignored by the DFLX
    command.

    The edge-flux constraints at the mid-side nodes are derived from  the
    magnetic vector potential A, which is related to the imposed magnetic
    flux B as follows:

    where r is the position of the mid-side node.

    The DFLX command creates a component named _DFLX for the constrained
    midside nodes. You can use this component to delete the constraints
    imposed by the DFLX command.

    This command is also valid in PREP7.
    """
    command = f"DFLX,{node},{bx},{by},{bz},{bx2},{by2},{bz2}"
    return self.run(command, **kwargs)


def dj(self, elem="", label="", value="", **kwargs):
    """Specifies boundary conditions on the components of relative motion of a

    APDL Command: DJ
    joint element.

    Parameters
    ----------
    elem
        Element number or ALL to be specified.

    label
        Valid labels are:

        UX - Displacement in local x direction.

        UY - Displacement in local y direction.

        UZ - Displacement in local z direction.

        ROTX - Rotation about local x axis.

        ROTY - Rotation about local y axis.

        ROTZ - Rotation about local y axis.

        VELX - Linear velocity in local x direction.

        VELY - Linear velocity in local y direction.

        VELZ - Linear velocity in local z direction.

        OMGX - Angular velocity in local x direction.

        OMGY - Angular velocity in local y direction.

        OMGZ - Angular velocity in local z direction.

        ACCX - Linear acceleration in local x direction.

        ACCY - Linear acceleration in local y direction.

        ACCZ - Linear acceleration in local z direction.

        DMGX - Angular acceleration in local x direction.

        DMGY - Angular acceleration in local y direction.

        DMGZ - Angular acceleration in local z direction.

    value
        Value of the label.

    Notes
    -----
    This command is valid for MPC184 joint elements. See DJDELE for
    information on deleting boundary conditions applied with the DJ
    command.

    You can apply only one displacement, velocity, or acceleration load at
    any relative degree of freedom. If multiple loads are specified, the
    last applied load overrides the previous ones. For example, the
    following commands apply loads to element 100:

    In this case, the velocity load (VELX) applied in the last command will
    override the displacement load (UX).

    Tabular boundary conditions (VALUE = %tabname%) can be used.

    %_FIX% is an ANSYS reserved table name. When VALUE is set to %_FIX%,
    ANSYS will prescribe the degree of freedom to the "current" relative
    displacement value. This option is only valid for the following labels:
    UX, UY, UZ, ROTX, ROTY, ROTZ. In most cases, %_FIX% usage is efficient
    and recommended for all structural degrees of freedom.
    """
    command = f"DJ,{elem},{label},{value}"
    return self.run(command, **kwargs)


def djdele(self, elem="", lab="", **kwargs):
    """Deletes boundary conditions on the components of relative motion of a

    APDL Command: DJDELE
    joint element.

    Parameters
    ----------
    elem
        Element number or ALL. ALL (or leaving this field blank) will
        delete all joint element boundary conditions specified by LAB.

    lab
        Valid labels are:

        UX - Displacement in local x direction.

        UY - Displacement in local y direction.

        UZ - Displacement in local z direction.

        ROTX - Rotation about local x axis.

        ROTY - Rotation about local y axis.

        ROTZ - Rotation about local z axis.

        VELX - Linear velocity in local x direction.

        VELY - Linear velocity in local y direction.

        VELZ - Linear velocity in local z direction.

        OMGX - Angular velocity in local x direction.

        OMGY - Angular velocity in local y direction.

        OMGZ - Angular velocity in local z direction.

        ACCX - Linear acceleration in local x direction.

        ACCY - Linear acceleration in local y direction.

        ACCZ - Linear acceleration in local z direction.

        DMGX - Angular acceleration in local x direction.

        DMGY - Angular acceleration in local y direction.

        DMGZ - Angular acceleration in local z direction.

        ALL, or (blank) - Delete all applied boundary conditions.

    Notes
    -----
    This command is valid for MPC184 joint elements. See DJ for information
    on specifying boundary conditions on the components of relative motion
    of a joint element.
    """
    command = f"DJDELE,{elem},{lab}"
    return self.run(command, **kwargs)


def djlist(self, elem="", **kwargs):
    """Lists boundary conditions applied to joint elements.

    APDL Command: DJLIST

    Parameters
    ----------
    elem
        Element number or ALL (or blank). Lists joint element boundary
        conditions on the specified element(s).

    Notes
    -----
    This command is valid for MPC184 joint elements. See DJ for information
    on specifying boundary conditions on joint elements.
    """
    command = f"DJLIST,{elem}"
    return self.run(command, **kwargs)


def dlist(self, node1="", node2="", ninc="", **kwargs):
    """Lists DOF constraints.

    APDL Command: DLIST

    Parameters
    ----------
    node1, node2, ninc
        List constraints for nodes NODE1 to NODE2 (defaults to NODE1) in
        steps of NINC (defaults to 1).  If ALL (default), NODE2 and NINC
        are ignored and constraints for all selected nodes [NSEL] are
        listed.  If NODE1 = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).  A
        component name may also be substituted for NODE1(NODE2 and NINC are
        ignored).

    Notes
    -----
    Listing applies to the selected nodes [NSEL] and the selected degree of
    freedom labels [DOFSEL].

    This command is valid in any processor.
    """
    command = f"DLIST,{node1},{node2},{ninc}"
    return self.run(command, **kwargs)


def dscale(self, rfact="", ifact="", tbase="", **kwargs):
    """Scales DOF constraint values.

    APDL Command: DSCALE

    Parameters
    ----------
    rfact
        Scale factor for the real component.  Zero (or blank) defaults to
        1.0.  Use a small number for a zero scale factor.

    ifact
        Scale factor for the imaginary component.  Zero (or blank) defaults
        to 1.0.  Use a small number for a zero scale factor.

    tbase
        Base temperature for temperature difference.  For temperatures, the
        scale factor is applied to the temperature difference (T-TBASE) and
        then added to TBASE.  T is the current temperature.

    Notes
    -----
    Scales degree of freedom constraint values (displacement, temperature,
    etc.) in the database.  If velocity and acceleration boundary
    conditions are applied in a structural analysis, they are also scaled
    by this command. Solid model boundary conditions are not scaled by this
    command, but boundary conditions on the FE model are scaled.

    Note:: : Such scaled FE boundary conditions may still be overwritten by
    unscaled solid model boundary conditions if a subsequent boundary
    condition transfer occurs.

    Scaling applies to the previously defined values for the selected nodes
    [NSEL] and the selected degree of freedom labels [DOFSEL].  Issue DLIST
    command to review results.

    DSCALE does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"DSCALE,{rfact},{ifact},{tbase}"
    return self.run(command, **kwargs)


def dsym(self, lab="", normal="", kcn="", **kwargs):
    """Specifies symmetry or antisymmetry degree-of-freedom constraints on

    APDL Command: DSYM
    nodes.

    Parameters
    ----------
    lab
        Symmetry label:

        SYMM - Generate symmetry constraints as described below (default).

        ASYM - Generate antisymmetry constraints as described below.

    normal
        Surface orientation label to determine the constraint set (surface
        is assumed to be perpendicular to this coordinate direction in
        coordinate system KCN):

        X - Surface is normal to coordinate X direction (default).  Interpreted as R
            direction for non-Cartesian coordinate systems.

        Y - Surface is normal to coordinate Y direction.   θ direction for non-Cartesian
            coordinate systems.

        Z - Surface is normal to coordinate Z direction.   Φ direction for spherical or
            toroidal coordinate systems.

    kcn
        Reference number of global or local coordinate system used to
        define surface orientation.

    Notes
    -----
    Specifies symmetry or antisymmetry degree-of-freedom constraints on the
    selected nodes. The nodes are first automatically rotated (any
    previously defined rotations on these nodes are redefined) into
    coordinate system KCN, then zero-valued constraints are generated, as
    described below, on the selected degree-of-freedom set (limited to
    displacement, velocity, and magnetic degrees of freedom) [DOFSEL].
    Constraints are defined in the (rotated) nodal coordinate system, as
    usual. See the D and NROTAT commands for additional details about
    constraints and nodal rotations.

    This command is also valid in PREP7.

    Symmetry or antisymmetry constraint generations are based upon the
    valid degrees of freedom in the model, i.e., the degrees of freedom
    associated with the elements attached to the nodes.  The labels for
    degrees of freedom used in the generation depend on the Normal label.

    For displacement degrees of freedom, the constraints generated are:

    For velocity degrees of freedom, the constraints generated are:

    For magnetic degrees of freedom, the SYMM label generates flux normal
    conditions (flux flows normal to the surface).  Where no constraints
    are generated, the flux normal condition is "naturally" satisfied.  The
    ASYM label generates flux parallel conditions (flux flows parallel to
    the surface).
    """
    command = f"DSYM,{lab},{normal},{kcn}"
    return self.run(command, **kwargs)


def dval(self, baseid="", lab="", value="", value2="", keycal="",
         **kwargs):
    """Defines values at enforced motion base.

    APDL Command: DVAL

    Parameters
    ----------
    baseid
        The identification number of the enforced motion base (defined
        using the D command in the modal analysis).

    lab
        U

        U - Enforced displacement.

        ACC - Enforced acceleration.

    value
        The value or table name reference for tabular boundary conditions.
        To specify a table, enclose the table name in percent (%) signs
        (DVAL,BaseID,U,%tablename%). Use the *DIM command to define a
        table.

    value2
        The value of the second degree of freedom (if present). If the
        analysis type and the degree of freedom allow a complex input,
        VALUE is the real component and VALUE2 is the imaginary component.

    keycal
        Displacement result calculation key:

        ON - Calculate absolute displacement and acceleration results (default).

        OFF - Calculate relative displacement and acceleration results.

    Notes
    -----
    In a mode-superposition harmonic or transient analysis, you can apply
    enforced displacement or acceleration loads. If multiple loads are
    specified for the same base identification number (BaseID), the last
    load applied overrides the previous ones. For example, the following
    commands apply displacement to the base with identification number 1:

    In this case, the acceleration (ACC) applied in the last command will
    override the displacement (U).

    Issue LSCLEAR,LSOPT to delete DVAL command options from the database.

    For more information, see Enforced Motion Method for Mode-Superposition
    Transient and Harmonic Analyses in the Structural Analysis Guide and
    Enforced Motion Method for Transient and Harmonic Analyses in the
    Mechanical APDL Theory Reference.
    """
    command = f"DVAL,{baseid},{lab},{value},{value2},{keycal}"
    return self.run(command, **kwargs)


def gsbdata(self, labz="", valuez="", labx="", valuex="", laby="",
            valuey="", **kwargs):
    """Specifies the constraints or applies the load at the ending point for

    APDL Command: GSBDATA
    generalized plane strain option.

    Parameters
    ----------
    labz
        Constraint or load at the ending point in the fiber Z direction.

        F - Apply a force in the fiber direction (default).

        LFIBER - Define a length change in the fiber direction.

    valuez
        Value for LabZ. The default is zero.

    labx
        Constraint or load on rotation about X.

        MX - Supply a moment to cause the rotation of the ending plane about X (default).

        ROTX - Define a rotation angle (in radians) of the ending plane about X.

    valuex
        Value for LabX. The default is zero.

    laby
        Constraint or load on rotation about Y

        MY - Supply a moment to cause the rotation of the ending plane about Y (default).

        ROTY - Define a rotation angle (in radians) of the ending plane about Y.

    valuey
        Value for LabY. The default is zero.

    Notes
    -----
    All inputs are in the global Cartesian coordinate system. For more
    information about the generalized plane strain feature, see Generalized
    Plane Strain Option of Current-Technology Solid Elements in the Element
    Reference.

    This command is also valid in PREP7.
    """
    command = f"GSBDATA,{labz},{valuez},{labx},{valuex},{laby},{valuey}"
    return self.run(command, **kwargs)


def gslist(self, lab="", **kwargs):
    """When using generalized plane strain, lists the input data or solutions.

    APDL Command: GSLIST

    Parameters
    ----------
    lab
        Specify the content to be listed.

        GEOMETRY - List the data input using GSGDATA

        BC - List the data input using GSBDATA.

        REACTIONS - When the command is issued in POST1, list the reaction force at the ending
                    point,

         and the moment about X and Y if the corresponding constraints were applied. - RESULTS

        When the command is issued in POST1, list the change of fiber length at the ending point during deformation and the rotation of the ending plane about X and Y during deformation. - ALL

    Notes
    -----
    This command can be used to list the initial position of the ending
    plane, the applied load or displacements in the fiber direction, the
    resulting position of the ending plane after deformation, and the
    available reaction forces and moments at the ending point.

    All inputs and outputs are in the global Cartesian coordinate system.
    For more information about the generalized plane strain feature, see
    Generalized Plane Strain Option of Current-Technology Solid Elements in
    the Element Reference.

    This command is valid in any processor.
    """
    command = f"GSLIST,{lab}"
    return self.run(command, **kwargs)



def morph(self, option="", remeshopt="", elemset="", armax="", voch="",
          arch="", step="", time="", stropt="", **kwargs):
    """Specifies morphing and remeshing controls.

    APDL Command: MORPH

    Parameters
    ----------
    option
        OFF - Turns off morphing for field elements (default).

        ON - Turns on morphing for field elements.

    remeshopt
        OFF - Do not remesh (default).

        ON - Remesh when element qualities fall below values
             specified by ARMAX, VOCH, or ARCH as explained
             below. Valid only when Option is ON.

    elemset
        ALL - Remesh all selected elements if the quality of the
              worst defined element falls below any quality
              requirement (default when Remeshopt = ON).

        CompName - Specify a component name, up to 32
                    characters. All elements included in this
                    component name are remeshed if the quality of
                    the worst element falls below any quality
                    requirement.

    armax
        The maximum allowable element generalized aspect ratio. Defaults to
        5.

    voch
        The maximum allowable change of element size (area or volume).
        Defaults to 3.

    arch
        The maximum allowable element aspect ratio change.  Defaults to 3.

    step
        The frequency of element quality checking, based on time steps. A
        quality check takes place at the intervals defined by STEP.
        Defaults to 1 (quality check at every step).

    time
        A quality check takes place at the time point specified. Defaults
        to -1 (a quality check at every time point).

    stropt
        NO

        NO - There are no structural elements in the model (default).

        YES - There are no structural elements in the model and the morphing happens after
              the structural solution.

    Notes
    -----
    MORPH is applicable to any non-structural field analysis (not including
    fluid elements). It activates displacement degrees of freedom for non-
    structural elements so that boundary conditions may be placed on the
    field mesh to constrain the movement of the non-structural mesh during
    morphing. It morphs the non-structural mesh using displacements
    transferred at the surface interface between the structural field and
    the non-structural field.  The displacements of non-structural elements
    are mesh displacements to avoid mesh distortion, but have no physical
    meaning except at the interface. MORPH does not support surface, link,
    or shell elements, or any element shape other than triangles, quads,
    tets, and bricks. Morphed fields must be in the global Cartesian system
    (CSYS = 0).

    After each remesh, new databases and results files are written with the
    extensions .rth0n and .db0n, where n is the remesh file number
    (FieldName.rth01, FieldName.rth02, ... and FieldName.db01,
    FieldName.db02, etc.).   The original database file is FieldName.dbo.
    The FieldName.db01, FieldName.db02, etc. files have elements that are
    detached from the solid model.

    Remeshing has the following restrictions:

    Valid only for the electrostatic elements (PLANE121, SOLID122, and
    SOLID123)

    Limited to triangle (2-D) and tetrahedral (3-D) options of these
    elements

     Valid only for the MFS solver

    No body loads allowed in the interior nodes of the remeshing domain

    Nodes on the boundary cannot be remeshed; remeshing will not work if
    morphing failed on the surface nodes

    Not suitable for extreme area or volume changes

    This command is also valid in SOLUTION.
    """
    command = f"MORPH,{option},,{remeshopt},{elemset},{armax},{voch},{arch},{step},{time},{stropt}"
    return self.run(command, **kwargs)


def damorph(self, area="", xline="", rmshky="", **kwargs):
    """Move nodes in selected areas to conform to structural displacements.

    APDL Command: DAMORPH

    Parameters
    ----------
    area
        Non-structural area to which mesh movement (morph) applies.  If
        ALL, apply morphing to all selected areas [ASEL].  If AREA = P,
        graphical picking is enabled.  A component may be substituted for
        AREA.

    xline
        Lines to be excluded from morphing.  If ALL, exclude all selected
        lines [LSEL] from morphing.  If XLINE = P, graphical picking is
        enabled.  A component may be substituted for XLINE.  If XLINE is
        blank (default), allow morphing of nodes attached to lines of the
        selected areas (AREA) which are not shared by unselected areas.
        See Notes for clarification.

    rmshky
        Remesh flag option:

        0 - Remesh the selected non-structural areas only if mesh morphing fails.

        1 - Remesh the selected non-structural areas and bypass mesh morphing.

        2 - Perform mesh morphing only and do not remesh.

    Notes
    -----
    The selected areas should include only non-structural regions adjacent
    to structural regions. DAMORPH will morph the non-structural areas to
    coincide with the deflections of the structural regions.

    Nodes in the structural regions move in accordance with computed
    displacements. Displacements from a structural analysis must be in the
    database prior to issuing DAMORPH.

    By default, nodes attached to lines can move along the lines, or off
    the lines (if a line is interior to the selected areas). You can use
    XLINE to restrain nodes on certain lines.

    By default (RMSHKEY = 0), DAMORPH will remesh the selected non-
    structural areas entirely if a satisfactory morphed mesh cannot be
    provided.

    If boundary conditions and loads are applied directly to nodes and
    elements, the DAMORPH command requires that these be removed before
    remeshing can take place.

    Exercise care with initial conditions defined by the IC command. Before
    a structural analysis is performed for a sequentially coupled analysis,
    the DAMORPH command requires that initial conditions be removed from
    all null element type nodes in the non-structural regions. Use ICDELE
    to delete the initial conditions.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"DAMORPH,{area},{xline},{rmshky}"
    return self.run(command, **kwargs)


def demorph(self, elem="", dimn="", rmshky="", **kwargs):
    """Move nodes in selected elements to conform to structural displacements.

    APDL Command: DEMORPH

    Parameters
    ----------
    elem
         Non-structural elements to which mesh movement (morph) applies.
        If ALL, apply morphing to all selected elements [ESEL]. If ELEM =
        P, graphical picking is enabled.  A component may be substituted
        for ELEM.

    dimn
        Problem dimensionality.  Use "2" for a 2-D problem and "3" for a
        3-D problem (no default).

    rmshky
        Remesh flag option:

        0 - Remesh the selected non-structural regions only if mesh morphing fails.

        1 - Remesh the selected non-structural regions and bypass mesh morphing.

        2 - Perform mesh morphing only and do not remesh.

    Notes
    -----
    The selected elements should include only non-structural regions
    adjacent to structural regions. The exterior nodes of the selected
    elements will usually be on the boundary of the region which will have
    node positions displaced. For DIMN = 2, elements must lie on a flat
    plane. The DEMORPH command requires a single domain grouping of
    elements be provided (multiple domains of elements are not permitted).
    Exterior nodes will be assumed fixed (no nodes will be morphed) unless
    they coincide with structural nodes having nonzero displacements.

    Nodes in the structural regions move in accordance with computed
    displacements. Displacements from a structural analysis must be in the
    database prior to issuing DEMORPH.

    By default (RMSHKY = 0), DEMORPH will remesh the selected non-
    structural regions entirely if a satisfactory morphed mesh cannot be
    provided.

    If boundary conditions and loads are applied directly to nodes and
    elements, the DEMORPH command requires that these be removed before
    remeshing can take place.

    Exercise care with initial conditions defined by the IC command. Before
    a structural analysis is performed for a sequentially coupled analysis,
    the DEMORPH command requires that initial conditions be removed from
    all null element type nodes in the non-structural regions. Use ICDELE
    to delete the initial conditions.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"DEMORPH,{elem},{dimn},{rmshky}"
    return self.run(command, **kwargs)


def dvmorph(self, volu="", xarea="", rmshky="", **kwargs):
    """Move nodes in selected volumes to conform to structural displacements.

    APDL Command: DVMORPH

    Parameters
    ----------
    volu
        Non-structural volume to which mesh movement (morph) applies.  If
        ALL, apply morphing to all selected volumes [VSEL]. If VOLU  = P,
        graphical picking is enabled. A component may be substituted for
        VOLU.

    xarea
        Areas to be excluded from morphing. If ALL, exclude all selected
        areas [ASEL].  If XAREA = P, graphical picking is enabled. A
        component may be substituted for XAREA. If XAREA is blank
        (default), allow morphing of nodes attached to areas of the
        selected volumes (VOLU) which are not shared by unselected volumes.
        (See Notes for clarification).

    rmshky
        Remesh flag option:

        0 - Remesh the selected non-structural volumes only if mesh morphing fails.

        1 - Remesh the selected non-structural volumes and bypass mesh morphing.

        2 - Perform mesh morphing only and do not remesh.

    Notes
    -----
    The selected volumes should include only non-structural regions
    adjacent to structural regions. DVMORPH will morph the non-structural
    volumes to coincide with the deflections of the structural regions.

    Nodes in the structural regions move in accordance with computed
    displacements. Displacements from a structural analysis must be in the
    database prior to issuing DVMORPH.

    By default, nodes attached to areas can move along the areas. You can
    use XAREA to restrain nodes on certain areas.

    By default (RMSHKY = 0), DVMORPH will remesh the selected non-
    structural volumes entirely if a satisfactory morphed mesh cannot be
    provided.

    If boundary conditions and loads are applied directly to nodes and
    elements, the DVMORPH command requires that these be removed before
    remeshing can take place.

    Exercise care with initial conditions defined by the IC command. Before
    a structural analysis is performed for a sequentially coupled analysis,
    the DVMORPH command requires that initial conditions be removed from
    all null element type nodes in the non-structural regions. Use ICDELE
    to delete the initial conditions.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"DVMORPH,{volu},{xarea},{rmshky}"
    return self.run(command, **kwargs)
